/**
 * @file Motor_Control.c
 * @author Ryan Lipski, Nicholas Sikkema
 * @version Revision: 1.0
 * @date Last Updated: 1/28/2015\n Created: 1/27/2015 4:45:33 PM
 * @brief This file is code for the control loops for the three motors.
 */ 

/** @brief The Z axis motor constant. */
#define motorZ_const 1
/** @brief The proportional gain Z. */
#define kpZ 2.0
/** @brief The integral gain Z. */
#define kiZ 2.0
/** @brief The derivative gain Z. */
#define kdZ 1.0
/** @brief Time difference for the velocity calculation. */
#define Velocity_Time_Difference 0.05
/** @brief Motor control type for the Z motor. */
#define Motor_Z_Control_Type 1

#include "../../Master_Build_Control.h"
#include "../../Subsystem/H_Bridge/DRV8830.h"
#include "../../Subsystem/IMU/LSM303.h"
#include "../../Subsystem/Pressure/MPX5010GP.h"
#include "Motor_Control.h"

#if Print_Motor_Information == 1
    #include <stdio.h>
    #include "../../Microcontroller/Communication/USART.h"
#endif

/* @brief Desired velocity of the x-axis motor. */
static volatile float v_desX = 0.0;
/* @brief Desired velocity of the x-axis motor. */
static volatile float v_desY = 0.0;
/* @brief Desired depth of the submarine. */
static volatile float p_desZ = 2.0;
static uint8_t Update_Control_Flag = 1;
static uint8_t Update_Motor_X_Flag = 0;
static uint8_t Update_Motor_Y_Flag = 0;
static uint8_t Update_Motor_Z_Flag = 0;

void Motor_Control_X_Open()
{
    DRV8830_Set_Voltage_Motor_X(v_desX/5.0*Motor_Max_Voltage);
}

void Motor_Control_Y_Open()
{
    DRV8830_Set_Voltage_Motor_Y(v_desY/5.0*Motor_Max_Voltage);
}

void Motor_Control_Z(float depth)
{
    float errZ = p_desZ - depth; //calculate the error in position
    #if Motor_Z_Control_Type == 0
        if(errZ>0.1)
        {
            DRV8830_Set_Voltage_Motor_Z(Motor_Max_Voltage/2);
        }
        else if(errZ<-0.1)
        {
            DRV8830_Set_Voltage_Motor_Z(-Motor_Max_Voltage);
        }
        else
        {
            DRV8830_Set_Voltage_Motor_Z(-Motor_Max_Voltage/2);
        }
    #elif Motor_Z_Control_Type == 1
        if(errZ<0.1 && errZ>-0.1)//dead band for depth
        {
            errZ=0;
        }
        static float errZ_old = 0;//old error signal for motor 1
        static float errZ_old_2 = 0;//old error signal for motor
        float rZ = kpZ*(errZ-errZ_old_2)+kiZ*(errZ+errZ_old_2)/2+kdZ*(errZ-2*errZ_old+errZ_old_2);
        errZ_old_2 = errZ_old;
        errZ_old = errZ;
        DRV8830_Set_Voltage_Motor_Z(rZ*motorZ_const);//set the motor 3's voltage using 5.14 * (VSET)/64 = Vout
        #if Print_Motor_Information == 1
            char Buffer[100] = {0x00};
            char Buffer_Depth[FLOATING_POINT_BUFFER_SIZE] = {0x00};
            char Buffer_Error[FLOATING_POINT_BUFFER_SIZE] = {0x00};
            char Buffer_Voltage[FLOATING_POINT_BUFFER_SIZE] = {0x00};
            sprintf(Buffer,"Depth: %s errZ %s zvoltage %s\r\n", USART_ftoa(depth, Buffer_Depth), USART_ftoa(errZ, Buffer_Error), USART_ftoa(rZ*motorZ_const, Buffer_Voltage));
            USART_Send_String(Buffer);
        #endif
    #endif
}

/**
 * @brief This function is the control loops for the three motors.
 */ 
void Motor_Control(void)
{
    if(Update_Motor_X_Flag)
    {
        Motor_Control_X_Open();
    }
    if(Update_Motor_Y_Flag)
    {
        Motor_Control_Y_Open();
    }
    if(Update_Motor_Z_Flag)
    {
        Motor_Control_Z(MPX5010GP_Depth()); 
    }
}

/**
 * @brief This function is the desired velocity for the X-axis motor.
 * @param Updated_Value The new desired velocity for the X-axis motor.
 */ 
void Motor_Control_Update_Velocity_X(float Updated_Value)
{
    v_desX = Updated_Value;
}

/**
 * @brief This function is the desired velocity for the Y-axis motor.
 * @param Updated_Value The new desired velocity for the Y-axis motor.
 */ 
void Motor_Control_Update_Velocity_Y(float Updated_Value)
{
    v_desY = Updated_Value;
}

/**
 * @brief @brief This function is the desired depth of the submarine.
 * @param Updated_Value The new desired depth for the submarine.
 */ 
void Motor_Control_Update_Depth_Z(float Updated_Value)
{
    p_desZ = Updated_Value;
}

/**
 * @brief This function controls the value for the update control motor x flag.
 * @param Input Value to set the update control motor x flag.
 */ 
void Motor_Control_Motor_X_Flag_Update(uint8_t Input)
{
    Update_Motor_X_Flag = Input;
}

/**
 * @brief This function controls the value for the update control motor y flag.
 * @param Input Value to set the update control motor y flag.
 */ 
void Motor_Control_Motor_Y_Flag_Update(uint8_t Input)
{
    Update_Motor_Y_Flag = Input;
}

/**
 * @brief This function controls the value for the update control motor z flag.
 * @param Input Value to set the update control motor z flag.
 */ 
void Motor_Control_Motor_Z_Flag_Update(uint8_t Input)
{
    Update_Motor_Z_Flag = Input;
}

/**
 * @brief This function is used to update the motor control if the update flag is enabled.
 */ 
void Motor_Control_Update(void)
{
    if(Update_Control_Flag)
    {
        Motor_Control();
        Update_Control_Flag = 0;
    }
}

/**
 * @brief This function is used to enable the control flag.
 */ 
void Motor_Control_Enable_Flag(void)
{
    Update_Control_Flag = 1;
}