/**
 * @file Swarming.c
 * @author Ryan Lipski, Nicholas Sikkema
 * @version Revision: 1.0
 * @date Last Updated: 4/29/2015\n Created: 2/12/2015 11:35:12 PM
 * @brief This file is code for the swarming algorithm.
 */

/** @brief The maximum ratio value of photodiodes to be considered "aligned." */
#define Photodiode_Ratio_Maximum 1.2
/** @brief The minimum ratio value of photodiodes to be considered "aligned." */
#define Photodiode_Ratio_Minimum 0.8

/** @brief The value associated with the various regions. */
enum Region_Value
{
    Red_Region_Value,
    Green_Region_Value,
    Yellow_Region_Value,
    Outside_Region_Value,
    Front_Region_Value,
    Center_Region_Value,
    Rear_Region_Value,
};

#include <stdint.h>
#include "../../Master_Build_Control.h"
#include "../../Subsystem/Light/CLS15.h"
#include "../../Algorithms/Motor_Control/Motor_Control.h"
#include "Swarming.h"

#if Print_Region == 1
    #include <stdio.h>
    #include "../../Microcontroller/Communication/USART.h"
#endif

/* X offset from the swarming code. */
static int X_Offset = 0;
/* Y offset from the swarming code. */
static int Y_Offset = 0;
/* Flag to signify that the swarming algorithm is to be updated. */
static int Update_Swarming_Flag = 1;

inline uint8_t Swarming_Region(int Current_Distance)
{
    if(Current_Distance>=Radius_Red)
    {
        return(Red_Region_Value);
    }
    else if(Current_Distance>=Radius_Green)
    {
        return(Green_Region_Value);
    }
    else if(Current_Distance>=Radius_Yellow)
    {
        return(Yellow_Region_Value);
    }
    else
    {
        return(Outside_Region_Value);
    }
}

inline uint8_t Swarming_Range_Left_Right(float Ratio)
{
    if(Ratio>Photodiode_Ratio_Maximum)
    {
        return(Front_Region_Value);
    }
    else if(Ratio<Photodiode_Ratio_Minimum)
    {
        return(Rear_Region_Value);
    }
    else
    {
        return(Center_Region_Value);
    }
}

inline void Swarming_Right_Calculation(int *X, int *Y, int Photodiode_Value_Front, int Photodiode_Value_Rear, float Ratio)
{
    uint8_t Ratio_Region = Swarming_Range_Left_Right(Ratio);
    uint8_t Front_Region = Swarming_Region(Photodiode_Value_Front);
    uint8_t Rear_Region = Swarming_Region(Photodiode_Value_Rear);
    uint8_t Check_Region = Outside_Region_Value;
    #if Print_Region == 1
        char Buffer[20] = {0x00};
        sprintf(Buffer, "Left %d, %d", Photodiode_Value_Front, Photodiode_Value_Rear);
        USART_Send_String(Buffer);
    #endif
    if(Front_Region <= Rear_Region)
    {
        Check_Region = Front_Region;
    }
    else
    {
        Check_Region = Rear_Region;
    }
    switch(Check_Region)
    {
        case Red_Region_Value: //check the near active region
            switch(Ratio_Region)
            {
                case Front_Region_Value:
                    //turn left b*Y a percentage and decrease forward speed b*Y a percentage
                    *X+=2;//left = +, right = -
                    #if Print_Region == 1
                        USART_Send_String("Right front red region\n\r");
                    #endif
                    break;
                case Center_Region_Value:
                    //turn left b*Y a percentage
                    *X+=1;//left = +, right = -
                    #if Print_Region == 1
                        USART_Send_String("Right center red region\n\r");
                    #endif
                    break;
                case Rear_Region_Value:
                    //turn left b*Y a percentage and increase forward speed b*Y a percentage
                    *X+=1;//left = +, right = -
                    *Y-=1;//forward = +, reverse = -
                    #if Print_Region == 1
                        USART_Send_String("Right rear red region\n\r");
                    #endif
                    break;
            }
            break;
        case Green_Region_Value:
            switch(Ratio_Region)
            {
                case Front_Region_Value:
                    //increase forward speed b*Y a percentage
                    *Y+=1;//forward = +, reverse = -
                    #if Print_Region == 1
                        USART_Send_String("Right front green region\n\r");
                    #endif
                    break;
                case Center_Region_Value:
                    #if Print_Region == 1
                        USART_Send_String("Right center green region\n\r");
                    #endif
                    break;
                case Rear_Region_Value:
                    //decrease forward speed b*Y a percentage
                    *Y-=1;//forward = +, reverse = -
                    #if Print_Region == 1
                        USART_Send_String("Right rear green region\n\r");
                    #endif
                    break;
            }
            break;
        case Yellow_Region_Value:
            switch(Ratio_Region)
            {
                case Front_Region_Value:
                    //turn right b*Y a percentage and decrease forward speed b*Y a percentage
                    *X+=1;//left = +, right = -
                    *Y+=1;//forward = +, reverse = -
                    #if Print_Region == 1
                        USART_Send_String("Right front yellow region\n\r");
                    #endif
                    break;
                case Center_Region_Value:
                    //turn right b*Y a percentage
                    *X+=1;//left = +, right = -
                    #if Print_Region == 1
                        USART_Send_String("Right center yellow region\n\r");
                    #endif
                    break;
                case Rear_Region_Value:
                    //turn right b*Y a percentage and increase forward speed b*Y a percentage
                    *X+=1;//left = +, right = -
                    *Y-=1;//forward = +, reverse = -
                    #if Print_Region == 1
                        USART_Send_String("Right rear yellow region\n\r");
                    #endif
                    break;
            }
            break;
        default:
            #if Print_Region == 1
                USART_Send_String("Right outside region\n\r");
            #endif
            break;
    }
}

inline void Swarming_Left_Calculation(int *X, int *Y, int Photodiode_Value_Front, int Photodiode_Value_Rear, float Ratio)
{
    uint8_t Ratio_Region = Swarming_Range_Left_Right(Ratio);
    uint8_t Front_Region = Swarming_Region(Photodiode_Value_Front);
    uint8_t Rear_Region = Swarming_Region(Photodiode_Value_Rear);
    uint8_t Check_Region = Outside_Region_Value;
    #if Print_Region == 1
        char Buffer[20] = {0x00};
        sprintf(Buffer, "Left %d, %d", Photodiode_Value_Front, Photodiode_Value_Rear);
        USART_Send_String(Buffer);
    #endif
    if(Front_Region <= Rear_Region)
    {
        Check_Region = Front_Region;
    }
    else
    {
        Check_Region = Rear_Region;
    }
    switch(Check_Region)
    {
        case Red_Region_Value: //check the near active region
            switch(Ratio_Region)
            {
                case Front_Region_Value:
                    //turn left b*Y a percentage and decrease forward speed b*Y a percentage
                    *X-=2;//left = +, right = -
                    *Y-=1;//forward = +, reverse = -
                    #if Print_Region == 1
                        USART_Send_String("Left front red region\n\r");
                    #endif
                    break;
                case Center_Region_Value:
                    //turn left b*Y a percentage
                    *X-=1;//left = +, right = -
                    #if Print_Region == 1
                        USART_Send_String("Left center red region\n\r");
                    #endif
                    break;
                case Rear_Region_Value:
                    //turn left b*Y a percentage and increase forward speed b*Y a percentage
                    *X-=1;//left = +, right = -
                    *Y+=1;//forward = +, reverse = -
                    #if Print_Region == 1
                        USART_Send_String("Left rear red region\n\r");
                    #endif
                    break;
            }
            break;
        case Green_Region_Value:
            switch(Ratio_Region)
            {
                case Front_Region_Value:
                    //increase forward speed b*Y a percentage
                    *Y+=1;//forward = +, reverse = -
                    #if Print_Region == 1
                        USART_Send_String("Left front green region\n\r");
                    #endif
                    break;
                case Center_Region_Value:
                    #if Print_Region == 1
                        USART_Send_String("Left center green region\n\r");
                    #endif
                    break;
                case Rear_Region_Value:
                    //decrease forward speed b*Y a percentage
                    *Y-=1;//forward = +, reverse = -
                    #if Print_Region == 1
                        USART_Send_String("Left rear green region\n\r");
                    #endif
                    break;
            }
            break;
        case Yellow_Region_Value:
            switch(Ratio_Region)
            {
                case Front_Region_Value:
                    //turn right b*Y a percentage and decrease forward speed b*Y a percentage
                    *X-=1;//left = +, right = -
                    *Y+=1;//forward = +, reverse = -
                    #if Print_Region == 1
                        USART_Send_String("Left front yellow region\n\r");
                    #endif
                    break;
                case Center_Region_Value:
                    //turn right b*Y a percentage
                    *X-=1;//left = +, right = -
                    #if Print_Region == 1
                        USART_Send_String("Left center yellow region\n\r");
                    #endif
                    break;
                case Rear_Region_Value:
                    //turn right b*Y a percentage and increase forward speed b*Y a percentage
                    *X-=1;//left = +, right = -
                    *Y-=1;//forward = +, reverse = -
                    #if Print_Region == 1
                        USART_Send_String("Left rear yellow region\n\r");
                    #endif
                    break;
            }
            break;
        default:
            #if Print_Region == 1
                USART_Send_String("Left outside region\n\r");
            #endif
            break;
    }
}

inline void Swarming_Forward_Calculation(int *X, int *Y, int Photodiode_Value)
{
    #if Print_Region == 1
        char Buffer[20] = {0x00};
        sprintf(Buffer, "Front %d", Photodiode_Value);
        USART_Send_String(Buffer);
    #endif
    switch(Swarming_Region(Photodiode_Value))
    {
        case Red_Region_Value: //check the near active region
            //turn left b*Y a percentage and reduce forward speed b*Y a percentage
            *X=5;//left = +, right = -
            *Y=0;//forward = +, reverse = -
            #if Print_Region == 1
                USART_Send_String("Front red region\n\r");
            #endif
            break;
        case Green_Region_Value: //check the far active region
            //turn left b*Y a percentage and reduce forward speed b*Y a percentage
            *X=5;//left = +, right = -
            *Y=0;//forward = +, reverse = -
            #if Print_Region == 1
                USART_Send_String("Front yellow region\n\r");
            #endif
            break;
        case Yellow_Region_Value: //check the far active region
            //turn left b*Y a percentage and reduce forward speed b*Y a percentage
            *X=5;//left = +, right = -
            *Y=0;//forward = +, reverse = -
            #if Print_Region == 1
            USART_Send_String("Front yellow region\n\r");
            #endif
            break;
    }
}

int Swarming_Retrieve_X_Offset(void)
{
    return(X_Offset);
}

int Swarming_Retrieve_Y_Offset(void)
{
    return(Y_Offset);
}

void Swarming(void)
{
    X_Offset = 0; //*X-plane motor control variable, left=+ and right=- movements
    Y_Offset = 3; //*Y-plane motor control variable, forward=+ and reverse=- movements
    Swarming_Left_Calculation(&X_Offset, &Y_Offset, CLS15_Retrieve_Photodiode_ADC_Left_Front(), CLS15_Retrieve_Photodiode_ADC_Left_Rear(), CLS15_Retrieve_Photodiode_Ratio_Left());
    Swarming_Right_Calculation(&X_Offset, &Y_Offset, CLS15_Retrieve_Photodiode_ADC_Right_Front(), CLS15_Retrieve_Photodiode_ADC_Right_Rear(), CLS15_Retrieve_Photodiode_Ratio_Right());
    Swarming_Forward_Calculation(&X_Offset, &Y_Offset, CLS15_Retrieve_Photodiode_ADC_Front());
}

void Swarming_Update(void)
{
    if(Update_Swarming_Flag)
    {
        Swarming();
        Motor_Control_Update_Velocity_X(Swarming_Retrieve_X_Offset());
        Motor_Control_Update_Velocity_Y(Swarming_Retrieve_Y_Offset());
        Motor_Control_Update_Depth_Z(2.0);
        Update_Swarming_Flag = 0;
    }
}

void Swarming_Enable_Flag(void)
{
    Update_Swarming_Flag = 1;
}