/**
 * @file State_Machine.c
 * @author Ryan Lipski, Nicholas Sikkema
 * @version Revision: 1.0
 * @date Last Updated: 4/29/2015\n Created: 2/12/2015 2:13:56 PM
 * @brief This file is code for the state machine code.
 */ 

#include "../../Algorithms/Motor_Control/Motor_Control.h"
#include "../../Algorithms/Navigation/Swarming.h"
#include "../../Master_Build_Control.h"
#include "../../Microcontroller/ADC/CD4051.h"
#include "../../Microcontroller/ADC/ADC.h"
#include "../../Microcontroller/Communication/I2C_Interface.h"
#include "../../Microcontroller/Interrupt/Timer.h"
#include "../../Subsystem/H_Bridge/DRV8830.h"
#include "../../Subsystem/Light/CLS15.h"
#include "../../Subsystem/Light/XPEBBL.h"
#include "../../Subsystem/IMU/LSM303.h"
#include "../../Subsystem/Pressure/MPX5010GP.h"
#include "../../Subsystem/Power/Battery.h"
#include "State_Machine.h"

#if Print_State == 1 || Print_Region == 1
    #include <stdio.h>
    #include "../../Microcontroller/Communication/USART.h"
#endif

static int Update_State_Flag = 1;

/** @brief Transition routine for between state 0 and 1. Initialize all active subsystems. */
inline void State_Machine_Initialization(void)
{
    #if Print_State == 1 || Print_Region == 1
        USART_Initialize();
        USART_Send_String("State transition 1\n\r");   
    #endif
    /* Initialize the TWI bus for the Compass and the H-bridge. */
    I2C_Interface_Initialize();
    /* Initialize the Compass subsystem. */
    //LSM303_Initialize();
    /* Initialize the h-bridges. */
    DRV8830_Initialize();
    /* Initialize the multiplexer. */
    CD4051_Initialize();
    /* Initialize the LEDs. */
    XPEBBL_Initialize();
    /* Initialize the timers. */
    Timer_Initialize();
    /* Disable the motor control for the X motor. */
    Motor_Control_Motor_X_Flag_Update(0);
    /* Disable the motor control for the Y motor. */
    Motor_Control_Motor_Y_Flag_Update(0);
    /* Disable the motor control for the Z motor. */
    Motor_Control_Motor_Z_Flag_Update(0);
    /* Disable Timer 0. */
    Timer0_Disable();
    /* Disable Timer 1. */
    Timer1_Disable();
    /* Enable Timer 2 to update the state machine. */
    Timer2_Enable();
}

/** @brief Routine for state 1. Indicate that the submarine is waiting to be depth. */
inline void State_Machine_State_1(void)
{
    #if Print_State == 1
        USART_Send_String("State 1\n\r");
    #endif
    /* Indicate by toggling the front LED. */
    XPEBBL_Toggle_Front_LED();
}

/** @brief Routine for state 2. Update the heading for going forward. */
inline void State_Machine_State_2(void)
{
    #if Print_State == 1
        USART_Send_String("State 2\n\r");
    #endif
    /* Update what is defined as the heading for going forward. */
    LSM303_UpdateForwardHeading();
    /* Update the motor control system. */
    Motor_Control_Update();
}

/** @brief Routine for state 3. Calibrate the offsets for the photodiodes. */
inline void State_Machine_State_3(void)
{
    #if Print_State == 1
        USART_Send_String("State 3\n\r");
    #endif
    /* Set the turning to maximum. */
    DRV8830_Set_Voltage_Motor_X(Motor_Max_Voltage);
    /* Read the photodiodes and update the offset if needed. */
    CLS15_Update_Photodiode_Offset();
    /* Update the motor control system. */
    Motor_Control_Update();
}

/** @brief Routine for state 4. Normal operation for the submarine. */
inline void State_Machine_State_4(void)
{
    #if Print_State == 1
        USART_Send_String("State 4\n\r");
    #endif
    /* Update the photodiodes taking into account the current active LED. */
    CLS15_Update_Photodiode_Values(XPEBLL_Change_LED());
    /* Update the swarming values using the current LED values. */
    Swarming_Update();
    /* Update the motor control system. */
    Motor_Control_Update();
}

/** @brief Routine for state 5. Toggling the front LED to indicate that . */
inline void State_Machine_State_5(void)
{
    #if Print_State == 1
        USART_Send_String("State 5\n\r");
    #endif
    /* Disable the turning motor. */
    DRV8830_Set_Voltage_Motor_X(0.0);
    /* Set the forward motor to maximum voltage. */
    DRV8830_Set_Voltage_Motor_Y(Motor_Max_Voltage);
    /* Update the front photodiodes taking into account the current active LED. */
    CLS15_Update_Photodiode_Values(XPEBBL_Toggle_Front_LED()<<5);
    /* Update the motor control system. */
    Motor_Control_Update();
}

/** @brief Routine for state 6. Indicate that the submarine is done. */
inline void State_Machine_State_6(void)
{
    #if Print_State == 1
        USART_Send_String("State 6\n\r");
    #endif
    /* Disable the turning motor. */
    DRV8830_Set_Voltage_Motor_X(0.0);
    /* Disable the forward motor. */
    DRV8830_Set_Voltage_Motor_Y(0.0);
    /* Indicate by toggling the front LED. */
    XPEBBL_Toggle_Front_LED();
    /* Update the motor control system. */
    Motor_Control_Update();
}

inline void State_Machine_Trasition_2(void)
{
    #if Print_State == 1
        USART_Send_String("State transition 2\n\r");
    #endif
    /* Turn off the LEDs. */
    XPEBBL_Set_Pin(0x00);
    /* Enable the motor control for the X motor. */
    Motor_Control_Motor_X_Flag_Update(0);
    /* Enable the motor control for the Y motor. */
    Motor_Control_Motor_Y_Flag_Update(0);
    /* Enable the motor control for the Z motor. */
    Motor_Control_Motor_Z_Flag_Update(1);
    /* Enable Timer 0 to update the motor control. */
    Timer0_Enable();
    /* Disable Timer 1. */
    Timer1_Disable();
}

inline void State_Machine_Trasition_3(void)
{
    #if Print_State == 1
        USART_Send_String("State transition 3\n\r");
    #endif
    /* Set the turning to maximum. */
    DRV8830_Set_Voltage_Motor_X(Motor_Max_Voltage);
    /* Enable the motor control for the X motor. */
    Motor_Control_Motor_X_Flag_Update(0);
    /* Enable the motor control for the Y motor. */
    Motor_Control_Motor_Y_Flag_Update(0);
    /* Enable the motor control for the Z motor. */
    Motor_Control_Motor_Z_Flag_Update(1);
    /* Enable Timer 0 to update the motor control. */
    Timer0_Enable();
    /* Disable Timer 1. */
    Timer1_Disable();
}

inline void State_Machine_Trasition_4(void)
{
    #if Print_State == 1
        USART_Send_String("State transition 4\n\r");
    #endif
    /* Enable the motor control for the X motor. */
    Motor_Control_Motor_X_Flag_Update(1);
    /* Enable the motor control for the Y motor. */
    Motor_Control_Motor_Y_Flag_Update(1);
    /* Enable the motor control for the Z motor. */
    Motor_Control_Motor_Z_Flag_Update(1);
    /* Enable Timer 0 to update the motor control. */
    Timer0_Enable();
    /* Enable Timer 0 to update the swarm algorithm. */
    Timer1_Enable();
}

void State_Machine_Trasition_5(void)
{    
    #if Print_State == 1
        USART_Send_String("State transition 5\n\r");
    #endif
    /* Disable the turning motor. */
    DRV8830_Set_Voltage_Motor_X(0.0);
    /* Set the forward motor to maximum voltage. */
    DRV8830_Set_Voltage_Motor_Y(Motor_Max_Voltage);
    /* Disable the motor control for the X motor. */
    Motor_Control_Motor_X_Flag_Update(0);
    /* Disable the motor control for the Y motor. */
    Motor_Control_Motor_Y_Flag_Update(0);
    /* Enable the motor control for the Z motor. */
    Motor_Control_Motor_Z_Flag_Update(1);
    /* Enable Timer 0 to update the motor control. */
    Timer0_Enable();
    /* Disable Timer 1. */
    Timer1_Disable();
}

void State_Machine_Trasition_6(void)
{
    #if Print_State == 1
        USART_Send_String("State transition 6\n\r");
    #endif
    /* Disable the turning motor. */
    DRV8830_Set_Voltage_Motor_X(0.0);
    /* Disable the forward motor. */
    DRV8830_Set_Voltage_Motor_Y(0.0);
    /* Set the motor control to surface. */
    Motor_Control_Update_Depth_Z(0);
    /* Disable the motor control for the X motor. */
    Motor_Control_Motor_X_Flag_Update(0);
    /* Disable the motor control for the Y motor. */
    Motor_Control_Motor_Y_Flag_Update(0);
    /* Enable the motor control for the Z motor. */
    Motor_Control_Motor_Z_Flag_Update(1);
    /* Enable Timer 0 to update the motor control. */
    Timer0_Enable();
    /* Disable Timer 1. */
    Timer1_Disable();
}

void State_Machine_Update(void)
{
    static int Current_State = 0;
    static int Previous_State = 0;
    if(Update_State_Flag)
    {
        switch(Current_State)
        {
            case 0:
                State_Machine_Initialization();
                #if State_Machine_Test_Mode > 0
                    Current_State = State_Machine_Test_Mode;
                #else
                    Current_State = 1;
                #endif
                break;
            case 1:
                State_Machine_State_1();
                #if Print_State == 1
                    char Buffer_State_1[30];
                    char Buffer_Depth[FLOATING_POINT_BUFFER_SIZE] = {0x00};
                    sprintf(Buffer_State_1, "%s ft\n\r",USART_ftoa(MPX5010GP_Depth(), Buffer_Depth));
                    USART_Send_String(Buffer_State_1);
                #endif
                if(MPX5010GP_Depth() > 0.5)
                {
                    Current_State = 2;
                }
                break;
            case 2:
                State_Machine_State_2();
                uint8_t IsStabilized = LSM303_IsStablized();
                #if Print_State == 1
                    if(IsStabilized)
                    {
                        USART_Send_String("Is stabilized");
                    }
                    else
                    {
                        USART_Send_String("Is not stabilized");
                    }
                #endif
                if(IsStabilized)
                {
                    Current_State = 3;
                }
                break;
            case 3:
                State_Machine_State_3();
                #if Print_State == 1
                    if(LSM303_RotationFinished())
                    {
                        USART_Send_String("Rotation is finished\n");
                    }
                    else
                    {
                        USART_Send_String("Rotation is not finished\n");
                    }
                #endif
                if(LSM303_RotationFinished())
                {
                    Current_State = 4;
                }
                break;
            case 4:
                State_Machine_State_4();
                uint8_t Motor_Status = DRV8830_Check_Motors();
                if(Motor_Status&0x02)
                {
                    Current_State = 6;
                }
                else if(Battery_Check_Status()||(Motor_Status&0x05))
                {
                    Current_State = 6;
                }
                break;
            case 5:
                State_Machine_State_5();
                if(CLS15_Retrieve_Photodiode_ADC_Front()<Radius_Red)
                {
                    Current_State = 6;
                }
                break;
            case 6:
                State_Machine_State_6();
                break;
        }
        #if State_Machine_Test_Mode == 0
            if(Current_State != Previous_State)
            {
                switch(Current_State)
                {
                    case 2:
                    State_Machine_Trasition_2();
                    break;
                    case 3:
                    State_Machine_Trasition_3();
                    break;
                    case 4:
                    State_Machine_Trasition_4();
                    break;
                    case 5:
                    State_Machine_Trasition_5();
                    break;
                    case 6:
                    State_Machine_Trasition_6();
                    break;
                }
            }
        #else
            if(Current_State != Previous_State)
            {
                switch(Current_State)
                {
                    case 2:
                    State_Machine_Trasition_2();
                    break;
                    case 3:
                    State_Machine_Trasition_3();
                    break;
                    case 4:
                    State_Machine_Trasition_4();
                    break;
                    case 5:
                    State_Machine_Trasition_5();
                    break;
                    case 6:
                    State_Machine_Trasition_6();
                    break;
                }
            }
            else
            {
                Current_State = Previous_State;
            }
        #endif
        Previous_State = Current_State;
        Update_State_Flag = 0;
        #if Print_State == 1
        char Buffer_State_4[100];
        char Buffer_State_4_2[FLOATING_POINT_BUFFER_SIZE];
        sprintf(Buffer_State_4, "Battery Status: %d, %s V\n\r", Battery_Check_Status(), USART_ftoa(ADC_Convert_To_Voltage(CLS15_Retrieve_Photodiode_ADC_Front()),Buffer_State_4_2));
        USART_Send_String(Buffer_State_4);
        #endif
    }
}

void State_Machine_Enable_Flag(void)
{
    Update_State_Flag = 1;
}