/**
 * @file ADC.c
 * @author Nicholas Sikkema
 * @version Revision: 1.5
 * @date Last Updated: 2/19/2015\n Created: 11/12/2014 7:36:35 PM
 * @brief This file is code to control the ADC on the ATMega328P.
 */ 

/** @brief The reference voltage for the ATMega328P. */
#define VREF 5.0

#include <avr/io.h>
#include "ADC.h"

/**
 * @brief This function sets up the ADC Pin that the function ADC_Read_Pin will read.
 * @param ADC_Pin Pin to be set up.\n 
 * 0000   | ADC0\n
 * 0001   | ADC1\n
 * 0010   | ADC2\n
 * 0011   | ADC3\n
 * 0100   | ADC4\n
 * 0101   | ADC5\n
 * 0110   | ADC6\n
 * 0111   | ADC7\n
 * 1000   | ADC8\n
 * 1001   | (reserved)\n
 * 1010   | (reserved)\n
 * 1011   | (reserved)\n
 * 1100   | (reserved)\n
 * 1101   | (reserved)\n
 * 1110   | 1.1V (V BG)\n
 * 1111   | 0V (GND)\n
 * @warning This function does not remove the upper 4 bits from the input.
 */ 
void ADC_Setup_Pin(unsigned char ADC_Pin) 
{
    /* Set up which analog input pin to use. */
    ADMUX = ADC_Pin;
    /* Use AVcc as the reference. */
    ADMUX |= (1<<REFS0);
    /* Clear for 10 bit resolution. */
    ADMUX &= ~(1<<ADLAR);
    /* Set the prescale to 128 for 8Mhz. */
    ADCSRA |= (1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
    /* Enable the ADC. */
    ADCSRA |= (1<<ADEN); 
}

/**
 * @brief Reads the selected pin from the function ADC_Setup_Pin.
 * @return The current ADC value from 0 to 1023.
 * @warning ADC_Setup_Pin needs to be ran before this function.
 */ 
int16_t ADC_Read_Pin(void) 
{
    /* Start the ADC conversion. */
    ADCSRA |= (1<<ADSC);
    /* Wait till the ADC complete flag is set. */
    while(ADCSRA&(1<<ADSC));
    /* Gather the lower byte of of the ADC value. */
    uint16_t ADC_Value = ADCL;
    /* Gather the upper byte of of the ADC value. */
    ADC_Value |= (ADCH<<8);
    /* Return the ADC value. */
    return(ADC_Value);
}

/**
 * @brief Converts the ADC value to flat
 * @param ADC_Value The current ADC value from 0 to 1023.
 * @return The current ADC value in voltage from 0 to Vref*1023/1024.
 */ 
float ADC_Convert_To_Voltage(int16_t ADC_Value)
{
    /* Convert the ADC value to voltage */
    return(((float)ADC_Value*VREF)/1024.0);
}