/**
 * @file CD4051.c
 * @author Nicholas Sikkema
 * @version Revision: 1.5
 * @date Last Updated: 1/22/2015\n Created: 10/23/2014 3:23:07 PM
 * @brief This file is code to control the CD4051 multiplexer connected to the ADC.
 */ 

#ifndef F_CPU
    /** @brief The current clock speed for the microcontroller. */
    #define F_CPU 16000000UL
#endif
/** @brief The default pin for the ADC on the ATMega328P. */
#define ADC_DEFAULT_PIN 0

#include <avr/io.h>
#include <util/delay.h>
#include "ADC.h"
#include "CD4051.h"

/** 
 * @brief This function is used to initialize the pins used for the multiplexer
 */
void CD4051_Initialize(void)
{
    /* Initialize ports B1, B2, and B3 as outputs. */
    DDRB|=(1<<DDB2)|(1<<DDB1)|(1<<DDB0);
    /* Initialize the default pin for the ADC. */
    ADC_Setup_Pin(ADC_DEFAULT_PIN);
}

/**
 * @brief This function is to read the selected multiplexer pin
 * Pin 0: Voltage\n
 * Pin 1: Photodiode Front\n
 * Pin 2: Photodiode Right Front\n
 * Pin 3: Photodiode Right Rear\n
 * Pin 4: Photodiode Left Front\n
 * Pin 5: Photodiode Left Rear\n
 * Pin 6: Photodiode Rear\n
 * Pin 7: Pressure Sensor\n
 * @param Multiplexer_Pin Pin to be selected from the multiplexer.
 * @return ADC value for the selected multiplexer pin.
 * @warning This function does not check the input for validity.
 */
int CD4051_Read_Single_Pin(unsigned char Multiplexer_Pin)
{
    /* Clear the lower three bits for PORTB. */
    PORTB &= 0xF8;
    /* Set the multiplexer pin. */
    PORTB |= Multiplexer_Pin;
    /* Have a delay to allow the switching of the multiplexer. */
    _delay_us(100);
    /* Return the ADC value for the current multiplexer pin. */
    return(ADC_Read_Pin());
}

/** 
 * @brief This function is used to calculate the distance for a photodiode.
 * @param Output An array holding ADC values for the selected multiplexer pins.
 * @param Minimum_Multiplexer_Pin Lowest pin range to be selected from the multiplexer.
 * @param Maximum_Multiplexer_Pin Highest pin range to be selected from the multiplexer.
 */
void CD4051_Read_Range(int *Output, unsigned char Minimum_Multiplexer_Pin, unsigned char Maximum_Multiplexer_Pin)
{
    /* Initialize the index for the array. */ 
    uint8_t Index = 0;
    /* Loop through the range between the minimum and maximum. */
    for(uint8_t i = Minimum_Multiplexer_Pin; i <= Maximum_Multiplexer_Pin; i++)
    {
        /* Read the selected pin and save at the current index. */
        Output[Index] = CD4051_Read_Single_Pin(i);
        /* Increment the current index. */
        Index++;
    }
}