/**
 * @file I2C_Interface.c
 * @author Nicholas Sikkema
 * @version Revision: 1.5
 * @date Last Updated: 2/3/2015\n Created: 10/30/2014 3:47:02 PM
 * @brief This file is code to control the TWI_Master.
 */ 

#ifndef F_CPU
    /** @brief The current clock speed for the microcontroller. */
    #define F_CPU 16000000UL
#endif

#include <stdint.h>
#include <avr/interrupt.h>
#include "I2C_Interface.h"
#include "TWI_Master.h"

/** 
 * @brief This function is to initialize the I2C interface.
 */
void I2C_Interface_Initialize()
{
    /* Initialize the TWI master. */
    TWI_Master_Initialise();
    /* Enable global interrupts. */
    sei();
}

/** 
 * @brief This function is to send a write I2C message over the I2C bus.
 * @param Device_Address The I2C device address.
 * @param Device_Register The register to write the data.
 * @param Value_To_Write Data to be sent over the bus.
 */
void I2C_Interface_Write(uint8_t Device_Address, uint8_t Device_Register, uint8_t Value_To_Write) 
{
    /* Initialize the message buffer. */
    uint8_t Message_Buff[3];
    /* Set array index 0 to the device address (bits 1-7) and the TWI_READ_BIT in write mode (bit 0). */
    Message_Buff[0] = (Device_Address<<1)|(FALSE<<TWI_READ_BIT);
    /* Set array index 1 to the device register you want to write to. */
    Message_Buff[1] = Device_Register;
    /* Set array index 2 to the value you want to write. */
    Message_Buff[2] = Value_To_Write;
    /* Send the write message over the I2C bus. */
    TWI_Start_Transceiver_With_Data(Message_Buff, 3);
}

/** 
 * @brief This function is to send and receive a read I2C message over the I2C bus for a single read.
 * @param Device_Address The I2C device address.
 * @param Device_Register The register to read the data.
 * @return The value from the register wanted.
 */
uint8_t I2C_Interface_Single_Read(uint8_t Device_Address, uint8_t Device_Register)
{
    /* Initialize a buffer to hold the information from the I2C bus. */
    uint8_t Message_Buff[2];
    /* Set array index 0 to the device address (bits 1-7) and the TWI_READ_BIT in write mode (bit 0). */
    Message_Buff[0] = (Device_Address<<1)|(FALSE<<TWI_READ_BIT);
    /* Set array index 1 to the device register you want to read from. */
    Message_Buff[1] = Device_Register;
    /* Send the I2C message to indicate specific location and sub. */
    TWI_Start_Transceiver_With_Data(Message_Buff, 2);
    /* Set array index 0 to the device address (bits 1-7) and the TWI_READ_BIT in read mode (bit 0). */
    Message_Buff[0] = (Device_Address<<1)|(TRUE<<TWI_READ_BIT);
    /* Send a dummy write to set the read pointer for the device. */
    TWI_Start_Transceiver_With_Data(Message_Buff, 2);
    /* Clear the message buffer. */
    Message_Buff[0] = 0;
    Message_Buff[1] = 0;
    /* Retrieve the data from the I2C bus. */
    TWI_Get_Data_From_Transceiver(Message_Buff, 2);
    /* Set the return buffer with the value from the I2C bus. */
    return(Message_Buff[1]);
}

/** 
 * @brief This function is to send and receive a read I2C message over the I2C bus for multiple reads.
 * @param Device_Address The I2C device address.
 * @param Device_Register The register to read the data.\n
 * Note that this function expects that the registers that you are reading from a continuous.
 * @param Return_Buff An array to store the data from the I2C bus.
 * @param Size_Of_Return_Buff Size of the return buffer array.
 */
void I2C_Interface_Read_Array(uint8_t Device_Address, uint8_t Device_Register, uint8_t *Return_Buff, uint8_t Size_Of_Return_Buff)
{
    /* Loop for the indexes. */
    for(int i=0; i<Size_Of_Return_Buff; i++)
    {
        /* Clear the return buffer. */
        Return_Buff[i] = 0;
        /* Set the return buffer with the value from the I2C bus. */
        Return_Buff[i] = I2C_Interface_Single_Read(Device_Address, Device_Register+i);
    }
}

/** 
 * @brief This function is to check if the I2C interface is not busy.
 * @return Boolean true (1) or false (0) for not being busy.
 */
uint8_t I2C_Interface_Not_Busy()
{
    /* Return the opposite of TWI_Transceiver_Busy(). */
    return(!TWI_Transceiver_Busy());
}