/**
 * Atmel Corporation
 * @file TWI_master.c
 * @author ltwa
 * Commented by: Nicholas Sikkema
 * @version Revision: 1.13
 * @date 24. mai 2004 11:31:20
 * @brief This is a sample driver for the TWI hardware modules.
 * It is interrupt driven. All functionality is controlled through
 * passing information to and from functions.
 */ 

#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "TWI_Master.h"

/* Transceiver buffer */
static uint8_t TWI_buf[TWI_BUFFER_SIZE];
/* Number of bytes to be transmitted. */
static uint8_t TWI_msgSize;// 
/* TWI state byte. Default set to TWI_NO_STATE. */
static uint8_t TWI_state = TWI_NO_STATE;// 

/* Status byte holding flags. */
static volatile uint8_t lastTransOK = 0;
/* Register for the TWI status. */
static volatile uint8_t TWI_statusReg = 0;

/**
 * @brief Call this function to set up the TWI master to its initial standby state.
 * @warning Remember to enable interrupts from the main application after initializing the TWI.
 */ 
void TWI_Master_Initialise(void)
{
    /* Set bit rate register (Baudrate). Defined in header file. */
    TWBR = TWI_TWBR;
    /* Set TWI prescaler. */
    TWSR = TWI_TWPS;
    /* Default content = SDA released. */
    TWDR = 0xFF;
    /* Enable TWI-interface and release TWI pins, Disable Interrupt, and No Signal requests. */
    TWCR = (1<<TWEN)|(0<<TWIE)|(0<<TWINT)|(0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC);
}

/**
 * @brief Call this function to test if the TWI_ISR is busy transmitting.
 * @return Status of the transceiver.
 */ 
uint8_t TWI_Transceiver_Busy(void)
{
    /* If the TWI Interrupt is enabled then the transceiver is busy. */
    return(TWCR&(1<<TWIE));
}

/**
 * @brief Call this function to fetch the state information of the previous operation. The function will hold execution (loop)
 * until the TWI_ISR has completed with the previous operation.
 * @return The TWI state code if there was an error.
 */ 
uint8_t TWI_Get_State_Info(void)
{
    /* Wait until TWI has completed the transmission. */
    while(TWI_Transceiver_Busy());
    /* Return the TWI state code if there was an error. */
    return TWI_state;
}

/**
 * @brief Call this function to send a prepared message. The first byte must contain the slave address and the
 * read/write bit. Consecutive bytes contain the data to be sent, or empty locations for data to be read
 * from the slave. Also include how many bytes that should be sent/read including the address byte.
 * The function will hold execution (loop) until the TWI_ISR has completed with the previous operation,
 * then initialize the next operation and return.
 * @param msg Message buffer to be sent over the I2C bus.
 * @param msgSize Size of the message buffer to be sent over the I2C bus.
 */
void TWI_Start_Transceiver_With_Data(uint8_t *msg, uint8_t msgSize)
{
    /* Wait until TWI is ready for next transmission. */
    while(TWI_Transceiver_Busy());// 
    
    /* Number of data to transmit. */
    TWI_msgSize = msgSize;
    /* Store slave address with R/W setting. */
    TWI_buf[0]  = msg[0];
    /* If it is a write operation, then also copy data. */
    if(!(msg[0]&(TRUE<<TWI_READ_BIT)))
    {
        /* Loop through the indexes of the message buffer. */
        for(uint8_t i = 1; i<msgSize; i++)
        {
            /* Copy index i of the msg buffer to index i of TWI_buf*/
            TWI_buf[i] = msg[i];
        }
    }
    
    /* Set the status register to zero*/
    TWI_statusReg = 0;
    /* Reset the TWI state. */
    TWI_state = TWI_NO_STATE;
    /* Set the TWCR to enable the TWI interface, enable TWI interrupt and clear the flag, and initiate the start condition. */
    TWCR = (1<<TWEN)|(1<<TWIE)|(1<<TWINT)|(0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)|(0<<TWWC);
}

/**
 * @brief Call this function to resend the last message. The driver will reuse the data previously put in the transceiver buffers.
 * The function will hold execution (loop) until the TWI_ISR has completed with the previous operation,
 * then initialize the next operation and return.
 */
void TWI_Start_Transceiver(void)
{
    /* Wait until TWI is ready for next transmission. */
    while(TWI_Transceiver_Busy());
    /* Set the status register to zero*/
    TWI_statusReg = 0;
    /* Reset the TWI state. */
    TWI_state = TWI_NO_STATE;
    /* Set the TWCR to enable the TWI interface, enable TWI interrupt and clear the flag, and initiate the start condition. */
    TWCR = (1<<TWEN)|(1<<TWIE)|(1<<TWINT)|(0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)|(0<<TWWC);
}

/**
 * @brief Call this function to read out the requested data from the TWI transceiver buffer. I.e. first call
 * TWI_Start_Transceiver to send a request for data to the slave. Then Run this function to collect the
 * data when they have arrived. Include a pointer to where to place the data and the number of bytes
 * requested (including the address field) in the function call. The function will hold execution (loop)
 * until the TWI_ISR has completed with the previous operation, before reading out the data and returning.
 * @param msg Message buffer to be received from the I2C bus.
 * @param msgSize Size of the message buffer to be received from the I2C bus.
 * @return 
 */
uint8_t TWI_Get_Data_From_Transceiver(uint8_t *msg, uint8_t msgSize)
{
    /* Wait until TWI is ready for next transmission. */
    while(TWI_Transceiver_Busy());
    /* Last transmission competed successfully. */
    if(lastTransOK)
    {
        /* Loop through the indexes of the message buffer. */
        for(uint8_t i = 0; i<msgSize; i++)
        {
            /* Copy index i of the TWI_buf buffer to index i of msg. */
            msg[i] = TWI_buf[i];
        }
    }
    /* Return TWI error code if there was an error in the previous transmission the function. */
    return lastTransOK;
}

/**
 * @brief This function is the Interrupt Service Routine (ISR), and called when the TWI interrupt is triggered;
 * that is whenever a TWI event has occurred. This function should not be called directly from the main
 * application.
 */
ISR(TWI_vect)
{
    /* Pointer index for the TWI buffer. */
    static uint8_t TWI_bufPtr;
    switch(TWSR)
    {
        /* Start has been transmitted. */
        case TWI_START:
        /* Repeated START has been transmitted. */
        case TWI_REP_START:
            /* Set buffer pointer to the TWI Address location. */
            TWI_bufPtr = 0;
        /* SLA+W has been transmitted and ACK received. */
        case TWI_MTX_ADR_ACK:
        /* Data byte has been transmitted and ACK received. */
        case TWI_MTX_DATA_ACK:
            /* Check if the pointer is less than last bit. */
            if(TWI_bufPtr < TWI_msgSize)
            {
                /* Update the TWDR with the transceiver buffer information and increment the pointer index*/
                TWDR = TWI_buf[TWI_bufPtr++];
                /* Set the TWCR to enable the TWI interface and enable TWI interrupt and clear the flag to send byte. */
                TWCR = (1<<TWEN)|(1<<TWIE)|(1<<TWINT)|(0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC);
            }
            else
            {
                /* Set status bits to completed successfully. */
                lastTransOK = TRUE;
                /* Set the TWCR to enable the TWI interface, disable TWI interrupt and clear the flag to send byte, and initiate a stop condition. */
                TWCR = (1<<TWEN)|(0<<TWIE)|(1<<TWINT)|(0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)|(0<<TWWC);
            }
            break;
        /* Data byte has been received and acknowledge transmitted. */
        case TWI_MRX_DATA_ACK:
            /* Update the transceiver buffer with the TWDR information. */
            TWI_buf[TWI_bufPtr++] = TWDR;
        /* SLA+R has been transmitted and acknowledge received. */
        case TWI_MRX_ADR_ACK:
            /* Check if the pointer is less than the second to last bit. */
            if(TWI_bufPtr < (TWI_msgSize-1))
            {
                /* Set the TWCR to enable the TWI interface, enable TWI Interrupt and clear the flag to read next byte, and send acknowledge after reception. */
                TWCR = (1<<TWEN)|(1<<TWIE)|(1<<TWINT)|(1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC);
            }
            else
            {
                /* Set the TWCR to enable the TWI interface, enable TWI Interrupt and clear the flag to read next byte, and send negative-acknowledge after reception. */
                TWCR = (1<<TWEN)|(1<<TWIE)|(1<<TWINT)|(0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC);
            }
            break;
        /* Data byte has been received and negative-acknowledge transmitted. */
        case TWI_MRX_DATA_NACK:
            /* Update the transceiver buffer with the TWDR information. */
            TWI_buf[TWI_bufPtr] = TWDR;
            /* Set status bits to completed successfully. */
            lastTransOK = TRUE;
            /* Set the TWCR to enable the TWI interface, disable TWI Interrupt and clear the flag, and initiate a stop condition. */
            TWCR = (1<<TWEN)|(0<<TWIE)|(1<<TWINT)|(0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)|(0<<TWWC);
            break;
        /* Arbitration lost on the I2C bus. */
        case TWI_ARB_LOST:
            /* Set the TWCR to enable the TWI interface, enable TWI Interrupt and clear the flag, and initiate a restart condition. */
            TWCR = (1<<TWEN)|(1<<TWIE)|(1<<TWINT)|(0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)|(0<<TWWC);
            break;
        /* Bus error due to an illegal start or stop condition. */
        case TWI_BUS_ERROR:
        /* SLA+W has been transmitted and negative-acknowledge received. */
        case TWI_MTX_ADR_NACK:
        /* SLA+R has been transmitted and negative-acknowledge received. */
        case TWI_MRX_ADR_NACK:
        /* Data byte has been transmitted and negative-acknowledge received. */
        case TWI_MTX_DATA_NACK:
        default:
            /* Store TWSR and automatically clears no Errors bit. */
            TWI_state = TWSR;
            /* Set the TWCR to enable the TWI interface and release TWI pins, disable TWI Interrupt flag, and initiate a stop condition. */
            TWCR = (1<<TWEN)|(0<<TWIE)|(1<<TWINT)|(0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)|(0<<TWWC);
    }
}