/**
 * Atmel Corporation
 * @file TWI_master.h
 * @author ltwa
 * @version Revision: 1.13
 * @date 24. mai 2004 11:31:22
 * @brief Header file for TWI_master.c.
 * Include this file in the application.
 */ 

#ifndef TWI_MASTER_H_
#define TWI_MASTER_H_

#include <stdint.h>

/** @brief Set this to the largest message size that will be sent including address byte. */
#define TWI_BUFFER_SIZE 16
/** @brief TWI_BITRATE in kHz. */
#define TWI_BITRATE 400

#ifndef F_CPU
    /** @brief The current clock speed for the microcontroller. */
    #define F_CPU 16000000UL
#endif

/** @brief TWI Bit rate Register setting. */
#define TWI_PSBR (F_CPU/2000/TWI_BITRATE-8)

#if TWI_PSBR > 0x3FC0
   #define TWI_TWPS 0x03
   #define TWI_TWBR 0xFF
   #define MESSAGE 'a'
#elif TWI_PSBR > 0x0FF0
   #define TWI_TWPS 0x03
   #define TWI_TWBR TWI_PSBR/64
   #define MESSAGE 'b'
#elif TWI_PSBR > 0x03FC
   #define TWI_TWPS 0x02
   #define TWI_TWBR TWI_PSBR/16
   #define MESSAGE 'c'
#elif TWI_PSBR > 0x00FF
   #define TWI_TWPS 0x01
   #define TWI_TWBR TWI_PSBR/4
   #define MESSAGE 'd'
#else
   #define TWI_TWPS 0x00
   #define TWI_TWBR TWI_PSBR
   #define MESSAGE 'e'
#endif

/** @brief Bit position for R/W bit in "address byte". */
#define TWI_READ_BIT 0
/** @brief Bit position for LSB of the slave address bits in the initial byte. */
#define TWI_ADR_BITS 1

/** @brief Boolean equivalent for TRUE. */
#define TRUE 1
/** @brief Boolean equivalent for FALSE. */
#define FALSE 0

/* General TWI Master status codes. */
/** @brief START has been transmitted. */
#define TWI_START 0x08
/** @brief Repeated START has been transmitted. */
#define TWI_REP_START 0x10
/** @brief Arbitration lost. */
#define TWI_ARB_LOST 0x38

/* TWI Master Transmitter status codes. */
/** @brief SLA+W has been tramsmitted and ACK received. */
#define TWI_MTX_ADR_ACK 0x18
/** @brief SLA+W has been tramsmitted and NACK received. */
#define TWI_MTX_ADR_NACK 0x20
/** @brief Data byte has been tramsmitted and ACK received. */
#define TWI_MTX_DATA_ACK 0x28
/** @brief Data byte has been tramsmitted and NACK received. */
#define TWI_MTX_DATA_NACK 0x30

/* TWI Master Receiver status codes. */
/** @brief SLA+R has been tramsmitted and ACK received. */
#define TWI_MRX_ADR_ACK 0x40
/** @brief SLA+R has been tramsmitted and NACK received. */
#define TWI_MRX_ADR_NACK 0x48
/** @brief Data byte has been received and ACK tramsmitted. */
#define TWI_MRX_DATA_ACK 0x50
/** @brief Data byte has been received and NACK tramsmitted. */
#define TWI_MRX_DATA_NACK 0x58

/* TWI Slave Transmitter status codes. */
/** @brief Own SLA+R has been received and ACK has been returned. */
#define TWI_STX_ADR_ACK 0xA8
/** @brief Arbitration lost in SLA+R/W as Master, own SLA+R has been received, and ACK has been returned. */
#define TWI_STX_ADR_ACK_M_ARB_LOST 0xB0
/** @brief Data byte in TWDR has been transmitted and ACK has been received. */
#define TWI_STX_DATA_ACK 0xB8
/** @brief Data byte in TWDR has been transmitted and NOT ACK has been received. */
#define TWI_STX_DATA_NACK 0xC0
/** @brief Last data byte in TWDR has been transmitted (TWEA = '0') ACK has been received. */
#define TWI_STX_DATA_ACK_LAST_BYTE 0xC8

/* TWI Slave Receiver status codes. */
/** @brief Own SLA+W has been received ACK has been returned. */
#define TWI_SRX_ADR_ACK 0x60
/** @brief Arbitration lost in SLA+R/W as Master, own SLA+W has been received, and ACK has been returned. */
#define TWI_SRX_ADR_ACK_M_ARB_LOST 0x68
/** @brief General call address has been received and ACK has been returned. */
#define TWI_SRX_GEN_ACK 0x70
/** @brief Arbitration lost in SLA+R/W as Master, General call address has been received, and ACK has been returned. */
#define TWI_SRX_GEN_ACK_M_ARB_LOST 0x78
/** @brief Previously addressed with own SLA+W, data has been received, and ACK has been returned. */
#define TWI_SRX_ADR_DATA_ACK 0x80
/** @brief Previously addressed with own SLA+W, data has been received, and NOT ACK has been returned. */
#define TWI_SRX_ADR_DATA_NACK 0x88
/** @brief Previously addressed with general call, data has been received, and ACK has been returned. */
#define TWI_SRX_GEN_DATA_ACK 0x90
/** @brief Previously addressed with general call, data has been received, and NOT ACK has been returned. */
#define TWI_SRX_GEN_DATA_NACK 0x98
/** @brief A STOP condition or repeated START condition has been received while still addressed as Slave. */
#define TWI_SRX_STOP_RESTART 0xA0

/* TWI Miscellaneous status codes. */
/** @brief No relevant state information available. */
#define TWI_NO_STATE 0xF8
/** @brief Bus error due to an illegal START or STOP condition. */
#define TWI_BUS_ERROR 0x00

void TWI_Master_Initialise(void);
uint8_t TWI_Transceiver_Busy(void);
uint8_t TWI_Get_State_Info(void);
void TWI_Start_Transceiver_With_Data(uint8_t *msg, uint8_t msgSize);
void TWI_Start_Transceiver(void);
uint8_t TWI_Get_Data_From_Transceiver(uint8_t *msg, uint8_t msgSize);

#endif