/**
 * @file USART.c
 * @author Ryan Lipski, Nick Sikkema
 * @version Revision: 2.0
 * @date Last Updated: 4/21/2015\n Created: 11/12/2014 7:36:35 PM
 * @brief This file is code to control the USART on the ATMega328P.
 */ 

/** @brief The current clock speed for the microcontroller. */
#define FOSC 16000000
/** @brief The desired baud rate for the USART connection. */
#define BAUD 76800
/** @brief The desired baud rate for the USART connection. */
#define MYUBRR (FOSC/(16UL*BAUD)-1)
/** @brief The desired precision for the conversion between float and string. */
#define PRECISION 3

#include <avr/io.h>
#include "USART.h"

/**
 * @brief Initializes USART on the ATMega328P.
 */ 
void USART_Initialize(void)
{
    /* Set baud rate. */
    UBRR0H = (unsigned char)(MYUBRR>>8);
    UBRR0L = (unsigned char)MYUBRR;
    /* Clear UCSR0A. This is to make sure that the USART is in single asynchronous mode. */
    UCSR0A = 0x00;
    /* Enable receiver and transmitter. */
    UCSR0B = (1<<RXEN0)|(1<<TXEN0);
    /* Set frame format: 8 data and 1 stop bit. */
    UCSR0C = (3<<UCSZ00);
}

/**
 * @brief Sends a single byte using USART.
 * @param data Character that is to be sent.
 */ 
void USART_Send_Byte(char data)
{
    /* Wait for empty transmit buffer. */
    while(!(UCSR0A & (1<<UDRE0)));
    /* Put data into buffer, sends the data. */
    UDR0 = data;
}

/**
 * @brief Receives a single byte using USART.
 * @return Character that is received.
 */ 
unsigned char USART_Receive_Byte(void)
{
    /* Wait for data to be received. */
    while(!(UCSR0A & (1<<RXC0)));
    /* Get and return received data from buffer. */
    return(UDR0);
}

/**
 * @brief Sends a c style string using USART.
 * @param string The c style string that is to be sent.
 */ 
void USART_Send_String(char* string)
{
    /* Loop through the string pointer. */
    for(char* pointer=string; *pointer!=0x00; pointer++) {
        /* Send the character. */
        USART_Send_Byte(*pointer);
    }
}

/**
 * @brief Receives a c style string using USART.
 * @param string The c style string that is to be received.
 * @return The number of characters that are received. 
 */ 
int USART_Read_String(char* string)
{
    /* A variable to keep track of the string length. */
    int count=0;
    while (1) {
        /** Receive the information from the keyboard and save it in a temporary variable. */
        char collector=USART_Receive_Byte();
        /* Break if the enter key is pressed. Check to see if the carriage return and line feed is sent. */
        if (collector=='\n'||collector=='\r'){
            break;
        }
        /* Update the string with the byte received. */
        *string++=collector;
        /* Increase the length count. */
        count++;
    }
    /* terminate the char array string with 0x00/ */
    *string=0x00;
    /* Return the length of the string. */
    return(count);
}

/**
 * @brief Receives a c style string using USART, then send the c style string.
 * @param string The c style string that is to be received.
 * @return The number of characters that are received.
 */
int USART_Read_String_With_Echo(char* string)
{
    /* A variable to keep track of the string length. */
    int count=0;
    while(1) {
        /** Receive the information from the keyboard and save it in a temporary variable. */
        char collector=USART_Receive_Byte();
        /* Send back the information just received to the screen. */
        USART_Send_Byte(collector);
        /* Break if the enter key is pressed. Check to see if the carriage return and line feed is sent. */
        if (collector=='\n'||collector=='\r'){
            break;
        }
        /* Update the string with the byte received. */
        *string++=collector;
        /* Increase the length count. */
        count++;
    }
    /* terminate the char array string with 0x00/ */
    *string=0x00;
    /* Return the length of the string. */
    return(count);
}

/**
 * @brief Converts a given floating point value to a string.
 * @param Input_Value The floating point value to be converted.
 * @param Buffer The buffer pointer for the character array. 
 * @return The floating point value in string form.
 */
char* USART_ftoa(float Input_Value, char* Buffer)
{
    /* Initialize a index variable. */
    unsigned int i = 0;
    /* Loop through the indexes and reinitialize the buffer. */
    for(i = 0; i<FLOATING_POINT_BUFFER_SIZE; i++)
    {
        /* Set the buffer index to NULL. */
        Buffer[i] = 0;
    }
    /* Copy the Input_Value to a local variable. Removing the decimal value. */
    int Temp_Value = Input_Value;
    /* Copy the Input_Value to a local variable. */
    float Temp_Input_Value = Input_Value;
    /* Reset the index pointer. */
    i = 0;
    /* Check to see if the Temp_Input_Value is negative. */
    if(Temp_Input_Value<0)
    {
        /* If the Input_Value is negative then flip sign. */
        Temp_Value = -Temp_Value;
        Temp_Input_Value = -Temp_Input_Value;
        /* Add a negative sign*/
        Buffer[i++] = '-';
    }
    /* Process individual digits */
    if (Temp_Value==0)
    {
        /* Add zero to the array. */
        Buffer[i++] = 48;
    }
    else
    {
        /* Copy the integer portion of the Input_Value to the Buffer. */
        while (Temp_Value != 0)
        {
            /* Add the current digit to the array. */
            Buffer[i++] = (char)(Temp_Value%10+48);
            /* Remove the last digit. */
            Temp_Value = Temp_Value/10;
        }    
    }
    /* Add the decimal place. */
    Buffer[i++] = '.';
    /* Reset the Temp_Value. */
    Temp_Value = (int)Temp_Input_Value;
    /* Initialize a precision counter. */
    int Precision_Count = 0;
    do {
        /* Update the floating portion by subtracting the integer portion. */
        Temp_Input_Value -= Temp_Value;
        /* Update the integer portion by shifting the decimal place and remove the decimal places. */
        Temp_Value = Temp_Input_Value*10;
        /* Append the new integer value to the Buffer. */
        Buffer[i++] = (char)(Temp_Value+48);
        /* Update the precision count. */
        Precision_Count++;
        /* Update the floating portion by shifting the decimal place. */
        Temp_Input_Value *= 10;
        /* Check to see if the index is greater than the buffer size or that the precision is good enough. */
    } while(i < FLOATING_POINT_BUFFER_SIZE && Precision_Count < PRECISION);
    /* Return the buffer pointer. */
    return(Buffer);
}