/**
 * @file DRV8830.c
 * @author Nicholas Sikkema
 * @version Revision: 2.0
 * @date Last Updated: 4/21/2015\n Created: 11/8/2014 3:54:57 PM
 * @brief This file is code to read control the h-bridges.
 */ 

/** @brief The conversion factor used between the floating point value and the value for the h-bridge. */
#define H_Bridge_Voltage_Conversion 12.5
/** @brief The h-bridge value that is equivalent to Motor_Max_Voltage. */
#define H_Bridge_Max_Voltage_VSET (uint8_t)(H_Bridge_Voltage_Conversion*Motor_Max_Voltage)
/** @brief The minimum voltage that the h-bridge will allow. */
#define H_Bridge_Min_Voltage 0.48
/** @brief The h-bridge value that is equivalent to Motor_Max_Voltage. */
#define H_Bridge_Min_Voltage_VSET (uint8_t)(H_Bridge_Voltage_Conversion*H_Bridge_Min_Voltage)
/** @brief The clear fault bit for the h-bridge. */
#define Clear_Fault_Bit 7
/** @brief Maximum amount of times allowed to fault before sending that the motor failed. */
#define Motor_Fault_Count_Max 5
/** @brief Maximum voltage jump allowed for a motor. */
#define Motor_Max_Jump 25

/** @brief I2C h-bridge addresses for the motors. */
enum Motor_Adress
{
    Motor_Address_X = 0x60,
    Motor_Address_Y = 0x62,
    Motor_Address_Z = 0x66
};

/** @brief The register addresses for the DRV8830. */
enum DRV8830_Registers
{
    DRV8830_Control = 0x00,
    DRV8830_Fault =  0x01,
};

/** @brief The list of motor modes for the DRV8830. */
enum DRV8830_Modes
{
    DRV8830_Standby = 0,
    DRV8830_Reverse =  1,
    DRV8830_Forward = 2,
    DRV8830_Stop = 3,
};

#include <stdio.h>
#include <avr/io.h>
#include "../../Master_Build_Control.h"
#include "../../Microcontroller/Communication/I2C_Interface.h"
#include "../../Microcontroller/Communication/USART.h"
#include "DRV8830.h"

/* Static variables to keep track of the last voltage that was sent to the h-bridge. */
static uint8_t Voltage_Value_X_Old = 0;
static uint8_t Voltage_Value_Y_Old = 0;
static uint8_t Voltage_Value_Z_Old = 0;

/* Static variables to keep track of the last mode that was sent to the h-bridge. */
static uint8_t Mode_X_Old = 0;
static uint8_t Mode_Y_Old = 0;
static uint8_t Mode_Z_Old = 0;

/* Static variables to keep track of the motors that are trying to start-up. */
static uint8_t Motor_X_Start_Up_Flag = 0;
static uint8_t Motor_Y_Start_Up_Flag = 0;
static uint8_t Motor_Z_Start_Up_Flag = 0;

/**
 * @brief This function converts a desired voltage to the equivalent voltage value for the h-bridge.
 * @param Voltage The desired voltage for the motor to be at.
 * @return The equivalent voltage value for the h-bridge.
 */
inline uint8_t DRV8830_Voltage_To_Int(float Voltage)
{
    /* Check to see if the input voltage is greater than the max rating of the motors that we are using. */
    if(Voltage>Motor_Max_Voltage)
    {
        /* To avoid burning up the motor if the input voltage is above the max voltage then return */
        return(H_Bridge_Max_Voltage_VSET);
    }
    /* Check to see if the input voltage is less than minimum voltage for the h-bridge. */
    else if(Voltage<H_Bridge_Min_Voltage)
    {
        /* If the input voltage is below the h-bridge minimum voltage, the only thing available is zero. */
        return(0);
    }
    /* Convert the voltage to a usable 8 bit value. */
    else
    {
        /* Return the converted h-bridge voltage value. */
        return((uint8_t)(Voltage*H_Bridge_Voltage_Conversion));
    }
}

/**
 * @brief This function checks the desired equivalent voltage value.
 * Checking against the previous mode and voltage, to make sure of three conditions.
 * First is that the new voltage is not significantly greater than the previous voltage.
 * Second is to set the flags for the start-up of the motor. This will be used to make sure that
 * not more one motor is being started up at the same time.
 * Third is to prevent the motors from switching polarity too quickly.
 * @param New_Voltage The desired equivalent voltage value of the h-bridge.
 * Note that the function updates this variable with a new voltage if it matches on of the three conditions.
 * @param New_Mode The desired mode of the h-bridge.
 * Note that the function updates this variable with a new mode if it matches on of the three conditions.
 * @param Old_Voltage The current equivalent voltage value of the h-bridge.
 * @param Old_Mode The mode of the h-bridge.
 * @return The start-up flag information for the motor.
 */
inline uint8_t DRV8830_Jump_Check(uint8_t *New_Voltage, uint8_t *New_Mode, uint8_t Old_Voltage, uint8_t Old_Mode)
{
    /* Initialize the variable that holds the start-up flag information. */
    uint8_t Return_Value = 0;
    /* Check the current mode of the h-bridge. */
    switch (Old_Mode)
    {
        /* The current mode of the h-bridge is reverse. */
        case DRV8830_Reverse:
            /* Check the desired mode of the h-bridge. */
            switch(*New_Mode)
            {
                /* The desired mode is reverse. */
                case DRV8830_Reverse:
                    /* Limit a positive voltage jump. */
                    if(*New_Voltage-Old_Voltage>Motor_Max_Jump)
                    {
                        /* Increase the voltage by the maximum jump. */
                        *New_Voltage = Old_Voltage+Motor_Max_Jump;
                    }
                    /* Limit a negative voltage jump. */
                    else if(Old_Voltage-*New_Voltage>Motor_Max_Jump)
                    {
                        /* Decrease the voltage by the maximum jump. */
                        *New_Voltage = Old_Voltage-Motor_Max_Jump;
                    }
                    break;
                /* The desired mode is forward. */
                case DRV8830_Forward:
                    /* Check to see if the motor voltage can be decreased without going to a complete stop. */
                    if(Old_Voltage>=Motor_Max_Jump+H_Bridge_Min_Voltage_VSET)
                    {
                        /* Decrease the motor voltage. */
                        *New_Voltage = Old_Voltage-Motor_Max_Jump;
                        *New_Mode = Old_Mode;
                    }
                    else
                    {
                        /* Stop the motor by setting the mode to stop and voltage to 0. */
                        *New_Voltage = 0;
                        *New_Mode = DRV8830_Stop;
                    }
                    break;
            }
            break;
        /* The current mode of the h-bridge is forward. */
        case DRV8830_Forward:
            /* Check the desired mode of the h-bridge. */
            switch(*New_Mode)
            {
                /* The desired mode is reverse. */
                case DRV8830_Reverse:
                    /* Check to see if the motor voltage can be decreased without going to a complete stop. */
                    if(Old_Voltage>=Motor_Max_Jump+H_Bridge_Min_Voltage_VSET)
                    {
                        /* Decrease the motor voltage. */
                        *New_Voltage = Old_Voltage-Motor_Max_Jump;
                        *New_Mode = Old_Mode;
                    }
                    else
                    {
                        /* Stop the motor by setting the mode to stop and voltage to 0. */
                        *New_Voltage = 0;
                        *New_Mode = DRV8830_Stop;
                    }
                    break;
                /* The desired mode is forward. */
                case DRV8830_Forward:
                    /* Limit a positive voltage jump. */
                    if(*New_Voltage-Old_Voltage>Motor_Max_Jump)
                    {
                        /* Increase the voltage by the maximum jump. */
                        *New_Voltage = Old_Voltage+Motor_Max_Jump;
                    }
                    /* Limit a negative voltage jump. */
                    else if(Old_Voltage-*New_Voltage>Motor_Max_Jump)
                    {
                        /* Decrease the voltage by the maximum jump. */
                        *New_Voltage = Old_Voltage-Motor_Max_Jump;
                    }
                    break;
            }
            break;
        /* The current mode of the h-bridge is stop. */
        default:
            /* Check the desired mode of the h-bridge. */
            switch(*New_Mode)
            {
                /* The desired mode is reverse. */
                case DRV8830_Reverse:
                    /* Set the start-up flag. */
                    Return_Value = 1;
                    /* Limit a positive voltage jump. */
                    if(*New_Voltage-Old_Voltage>Motor_Max_Jump)
                    {
                        /* Increase the voltage by the maximum jump. */
                        *New_Voltage = Old_Voltage+Motor_Max_Jump;
                    }
                    break;
                /* The desired mode is forward. */
                case DRV8830_Forward:
                    /* Set the start-up flag. */
                    Return_Value = 1;
                    /* Limit a positive voltage jump. */
                    if(*New_Voltage-Old_Voltage>Motor_Max_Jump)
                    {
                        /* Increase the voltage by the maximum jump. */
                        *New_Voltage = Old_Voltage+Motor_Max_Jump;
                    }
                    break;
            }
            break;
    }
    /* Return the start-up flag information. */
    return(Return_Value);
}

/**
 * @brief This function uses the a desired voltage to determine the mode for the h-bridge.
 * @param Voltage The desired voltage for the motor to be at.
 * @return The new mode for the h-bridge.
 */
inline uint8_t DRV8830_Mode(float *Voltage)
{
    /* Initialize the variable that holds mode information. */
    uint8_t Return_Value = DRV8830_Stop;
    /* Check to see if the voltage is positive or negative. */
    if(*Voltage<0.0)
    {
        /* Invert the voltage if the voltage is negative. */
        *Voltage*=-1;
        /* Check to see if the voltage is above or equal to the minimum voltage. */
        if(*Voltage>=H_Bridge_Min_Voltage)
        {
            /* Set the motor mode to reverse. */
            Return_Value = DRV8830_Reverse;
        }
    }
    else
    {
        /* Check to see if the voltage is above or equal to the minimum voltage. */
        if(*Voltage>=H_Bridge_Min_Voltage)
        {
            /* Set the motor mode to forward. */
            Return_Value = DRV8830_Forward;
        }
    }
    /* Return the mode information. */
    return(Return_Value);
}

/**
 * @brief This function checks whether or not a motor should be updated.
 * Making sure that there are no motors that are turning on at the same time.
 * @param Motor_Address The motor address of the motor to be updated.
 * @return Whether or not the motor should update.
 */
uint8_t DRV8830_Start_Up_Flag_Control(uint8_t Motor_Address)
{
    /* Initialize the variable that holds the information needed for updating the motor. */
    uint8_t Return_Value = 0;
    /* Variables to keep track of which motor is starting up. */
    static uint8_t Start_Up_X = 0;
    static uint8_t Start_Up_Y = 0;
    static uint8_t Start_Up_Z = 0;
    /* Check the motor address. */
    switch(Motor_Address)
    {
        /* Check the information for the X motor. */
        case Motor_Address_X:
            /* Check if the routine wants to start-up the motor. */
            if(Motor_X_Start_Up_Flag)
            {
                /* Check if another motor is starting up. */
                if(Start_Up_Y||Start_Up_Z)
                {
                    /* Keep the motor off due to another motor starting. */
                    Start_Up_X = 0;
                }
                else
                {
                    /* Allow the motor to update. */
                    Return_Value = 1;
                    /* Start-up the motor. */
                    Start_Up_X = 1;
                }
            }
            else
            {
                /* Normal operation of the motor. */
                Return_Value = 1;
                /* Clear the startup flag. */
                Start_Up_X = 0;
            }
            break;
        /* Check the information for the Y motor. */
        case Motor_Address_Y:
            /* Check if the routine wants to start-up the motor. */
            if(Motor_Y_Start_Up_Flag)
            {
                /* Check if another motor is starting up. */
                if(Start_Up_X||Start_Up_Z)
                {
                    /* Keep the motor off due to another motor starting and clear the start-up variable. */
                    Start_Up_Y = 0;
                }
                else
                {
                    /* Allow the motor to update. */
                    Return_Value = 1;
                    /* Start-up the motor. */
                    Start_Up_Y = 1;
                }
            }
            else
            {
                /* Normal operation of the motor. */
                Return_Value = 1;
                /* Clear the startup flag. */
                Start_Up_Y = 0;
            }
            break;
        /* Check the information for the Z motor. */
        case Motor_Address_Z:
            /* Check if the routine wants to start-up the motor. */
            if(Motor_Z_Start_Up_Flag)
            {
                /* Check if another motor is starting up. */
                if(Start_Up_X||Start_Up_Y)
                {
                    /* Keep the motor off due to another motor starting and clear the start-up variable. */
                    Start_Up_Z = 0;
                }
                else
                {
                    /* Allow the motor to update. */
                    Return_Value = 1;
                    /* Start-up the motor. */
                    Start_Up_Z = 1;
                }
            }
            else
            {
                /* Normal operation of the motor. */
                Return_Value = 1;
                /* Clear the startup flag. */
                Start_Up_Z = 0;
            }
            break;
    }
    /* Return the information needed for updating the motor. */
    return(Return_Value);
}

/** 
 * @brief This function sets the output voltage for the h-bridge.
 * @param Address The h-bridge I2C address.
 * @param Voltage_Value The equivalent voltage value for the h-bridge. Note this is not the desired voltage.
 * @param Mode The direction for the h-bridge (0 Standby/coast, 1 Reverse, 2 Forward, 3 Brake).
 * @warning This function does not check if the mode is above 3 (the maximum for the specified selection).
 */
inline void DRV8830_Set_Voltage(uint8_t Address, uint8_t Voltage_Value, uint8_t Mode)
{
    /* Send the update voltage message to the h-bridge. */
    I2C_Interface_Write(Address, DRV8830_Control, (Voltage_Value<<2)|Mode);
}

/** 
 * @brief This function sets the output voltage for the X-axis h-bridge. Note that this function only Updates the h-bridge if the mode or voltage is different.
 * @param Voltage The desired voltage for the motor to be at.
 */
void DRV8830_Set_Voltage_Motor_X(float Voltage)
{
    uint8_t Mode = DRV8830_Mode(&Voltage);
    /* Convert the voltage value to a h-bridge value.*/
    uint8_t Voltage_Value = DRV8830_Voltage_To_Int(Voltage);
    /* Check to if the voltage value and the mode are the same as the previous values. */
    if (Voltage_Value_X_Old != Voltage_Value || Mode_X_Old != Mode)
    {
		/* Check the start up flag using the new values for the X motors. */
        Motor_X_Start_Up_Flag = DRV8830_Jump_Check(&Voltage_Value, &Mode, Voltage_Value_X_Old, Mode_X_Old);
		/* Check the check the start up flags. */
        if(DRV8830_Start_Up_Flag_Control(Motor_Address_X))
        {
            /* Update the h-bridge voltage value for the X-axis motor. */
            DRV8830_Set_Voltage(Motor_Address_X, Voltage_Value, Mode);
            /* Update the previous voltage value for the X-axis motor. */
            Voltage_Value_X_Old = Voltage_Value;
            /* Update the previous mode for the X-axis motor. */
            Mode_X_Old = Mode;
        }
        #if H_Bridge_Print == 1  
            /* Initialize a character array. */
            char Buffer[50];
            /* Setup the buffer to send. */
            sprintf(Buffer, "X V %u, M %u, F %u\r\n", Voltage_Value_X_Old, Mode_X_Old, Motor_X_Start_Up_Flag);
            /* Send the string over USART. */
            USART_Send_String(Buffer);
        #endif
    }
}

/** 
 * @brief This function sets the output voltage for the Y-axis h-bridge. Note that this function only Updates the h-bridge if the mode or voltage is different.
 * @param Voltage The desired voltage for the motor to be at.
 */
void DRV8830_Set_Voltage_Motor_Y(float Voltage)
{
    uint8_t Mode = DRV8830_Mode(&Voltage);
    /* Convert the voltage value to a h-bridge value.*/
    uint8_t Voltage_Value = DRV8830_Voltage_To_Int(Voltage);
    /* Check to if the voltage value and the mode are the same as the previous values. */
    if (Voltage_Value_Y_Old != Voltage_Value || Mode_Y_Old != Mode)
    {
		/* Check the start up flag using the new values for the Y motors. */
        Motor_Y_Start_Up_Flag = DRV8830_Jump_Check(&Voltage_Value, &Mode, Voltage_Value_Y_Old, Mode_Y_Old);
		/* Check the check the start up flags. */
        if(DRV8830_Start_Up_Flag_Control(Motor_Address_Y))
        {
            /* Update the h-bridge voltage value for the X-axis motor. */
            DRV8830_Set_Voltage(Motor_Address_Y, Voltage_Value, Mode);
            /* Update the previous voltage value for the X-axis motor. */
            Voltage_Value_Y_Old = Voltage_Value;
            /* Update the previous mode for the X-axis motor. */
           Mode_Y_Old = Mode;
        }
        #if H_Bridge_Print == 1  
            /* Initialize a character array. */
            char Buffer[50];
            /* Setup the buffer to send. */
            sprintf(Buffer, "Y V %u, M %u, F %u\r\n", Voltage_Value_Y_Old, Mode_Y_Old, Motor_Y_Start_Up_Flag);
            /* Send the string over USART. */
            USART_Send_String(Buffer);
        #endif
    }
}

/** 
 * @brief This function sets the output voltage for the Z-axis h-bridge. Note that this function only Updates the h-bridge if the mode or voltage is different.
 * @param Voltage The desired voltage for the motor to be at.
 */
void DRV8830_Set_Voltage_Motor_Z(float Voltage)
{
    uint8_t Mode = DRV8830_Mode(&Voltage);
    /* Convert the voltage value to a h-bridge value.*/
    uint8_t Voltage_Value = DRV8830_Voltage_To_Int(Voltage);
    /* Check to if the voltage value and the mode are the same as the previous values. */
    if (Voltage_Value_Z_Old != Voltage_Value || Mode_Z_Old != Mode)
    {
		/* Check the start up flag using the new values for the Z motors. */
        Motor_Z_Start_Up_Flag = DRV8830_Jump_Check(&Voltage_Value, &Mode, Voltage_Value_Z_Old, Mode_Z_Old);
		/* Check the check the start up flags. */
        if(DRV8830_Start_Up_Flag_Control(Motor_Address_Z))
        {
            /* Update the h-bridge voltage value for the X-axis motor. */
            DRV8830_Set_Voltage(Motor_Address_Z, Voltage_Value, Mode);
            /* Update the previous voltage value for the X-axis motor. */
            Voltage_Value_Z_Old = Voltage_Value;
            /* Update the previous mode for the X-axis motor. */
            Mode_Z_Old = Mode;
        }
        #if H_Bridge_Print == 1  
            /* Initialize a character array. */
            char Buffer[50];
            /* Setup the buffer to send. */
            sprintf(Buffer, "Z V %u, M %u, F %u\r\n", Voltage_Value_Z_Old, Mode_Z_Old, Motor_Z_Start_Up_Flag);
            /* Send the string over USART. */
            USART_Send_String(Buffer);
        #endif
    }
}

/** 
 * @brief This function sets the initial output voltage for all three h-bridges.
 */
inline void DRV8830_Initialize(void)
{
    /* Initialize the X-axis motor to 0 volts. */
    DRV8830_Set_Voltage_Motor_X(0.0);
    /* Initialize the Y-axis motor to 0 volts. */
    DRV8830_Set_Voltage_Motor_Y(0.0);
    /* Initialize the Z-axis motor to 0 volts. */
    DRV8830_Set_Voltage_Motor_Z(0.0);
}

/** 
 * @brief This function reads the fault byte for the h-bridge.
 * @param Address The h-bridge I2C address.
 * @return The current fault byte for the h-bridge.
 */
inline uint8_t DRV8830_Read_Fault(uint8_t Address)
{
    /* Read and send back the contents of the fault byte. */
    return(I2C_Interface_Single_Read(Address, DRV8830_Fault));
}

/** 
 * @brief This function reads the fault byte for the X-axis h-bridge.
 * @return The current fault byte for the X-axis h-bridge.
 */
uint8_t DRV8830_Read_Fault_Motor_X(void)
{
    /* Return the fault byte for the X-axis motor. */
    return(DRV8830_Read_Fault(Motor_Address_X));
}

/** 
 * @brief This function reads the fault byte for the Y-axis h-bridge.
 * @return The current fault byte for the Y-axis h-bridge.
 */
uint8_t DRV8830_Read_Fault_Motor_Y(void)
{
    /* Return the fault byte for the Y-axis motor. */
    return(DRV8830_Read_Fault(Motor_Address_Y));
}

/** 
 * @brief This function reads the fault byte for the Z-axis h-bridge.
 * @return The current fault byte for the Z-axis h-bridge.
 */
uint8_t DRV8830_Read_Fault_Motor_Z(void)
{
    /** Return the fault byte for the Z-axis motor. */
    return(DRV8830_Read_Fault(Motor_Address_Z));
}

/** 
 * @brief This function clears the fault byte for the h-bridge.
 * @param Address The h-bridge I2C address.
 */
inline void DRV8830_Clear_Fault(uint8_t Address)
{
    /* Send the I2C message to the h-bridge to clear the fault bit. */
    I2C_Interface_Write(Address, DRV8830_Fault, 1<<Clear_Fault_Bit);
}

/** 
 * @brief This function clears the fault byte for the X-axis h-bridge.
 */
void DRV8830_Clear_Fault_Motor_X(void)
{
    /* Clear the fault byte for the X-axis motor. */
    DRV8830_Clear_Fault(Motor_Address_X);
}

/** 
 * @brief This function clears the fault byte for the Y-axis h-bridge.
 */
void DRV8830_Clear_Fault_Motor_Y(void)
{
    /* Clear the fault byte for the Y-axis motor. */
    DRV8830_Clear_Fault(Motor_Address_Y);
}

/** 
 * @brief This function clears the fault byte for the Z-axis h-bridge.
 */
void DRV8830_Clear_Fault_Motor_Z(void)
{
    /* Clear the fault byte for the Z-axis motor. */
    DRV8830_Clear_Fault(Motor_Address_Z);
}


void DRV8830_Reset_Motor_X(void)
{
    /* Clear fault previous fault values. */
    DRV8830_Clear_Fault_Motor_X();
    /* Set the motor voltage to 0. */
    DRV8830_Set_Voltage_Motor_X(0.0);
}

void DRV8830_Reset_Motor_Y(void)
{
    /* Clear fault previous fault values. */
    DRV8830_Clear_Fault_Motor_Y();
    /* Set the motor voltage to 0. */
    DRV8830_Set_Voltage_Motor_Y(0.0);
}

void DRV8830_Reset_Motor_Z(void)
{
    /* Clear fault previous fault values. */
    DRV8830_Clear_Fault_Motor_Z();
    /* Set the motor voltage to 0. */
    DRV8830_Set_Voltage_Motor_Z(0.0);
}

/** 
 * @brief This function checks to see if the X-axis motor is failing. 
 * Counting and resetting the motor faults to make sure the motor is able to continue. 
 * @return Whether or not the fault count is greater than a specified amount.
 */
uint8_t DRV8830_Check_Motor_X(void)
{
    /* Initialize the fault count for amount of motor failures. */
    static uint8_t Fault_Count_X = 0;
    /* Read the fault register from the X-axis motor h-bridge. */
    uint8_t Fault_Value = DRV8830_Read_Fault_Motor_X();
    if(Fault_Value&0x01)
    {
        /* Clear the fault register for the X-axis motor h-bridge. */
        DRV8830_Reset_Motor_X();
        /* Increment the fault count for the Y-axis motor. */
        Fault_Count_X++;
    }
    else if(Fault_Value)
    {
        /* Clear fault previous fault values. */
        DRV8830_Clear_Fault_Motor_X();
    }
    else
    {
        /* Clear the fault count for the X-axis motor. */
        Fault_Count_X = 0;
    }
    /* Return whether or not the fault count is greater than a specified amount. */
    return(Fault_Count_X>Motor_Fault_Count_Max);
}

/** 
 * @brief This function checks to see if the Y-axis motor is failing. 
 * Counting and resetting the motor faults to make sure the motor is able to continue. 
 * @return Whether or not the fault count is greater than a specified amount.
 */
uint8_t DRV8830_Check_Motor_Y(void)
{
    /* Initialize the fault count for amount of motor failures. */
    static uint8_t Fault_Count_Y = 0;
    /* Read the fault register from the Y-axis motor h-bridge. */
    uint8_t Fault_Value = DRV8830_Read_Fault_Motor_Y();
    if(Fault_Value&0x01)
    {
        /* Clear the fault register for the Y-axis motor h-bridge. */
        DRV8830_Reset_Motor_Y();
        /* Increment the fault count for the Y-axis motor. */
        Fault_Count_Y++;
    }
    else if(Fault_Value)
    {
        /* Clear fault previous fault values. */
        DRV8830_Clear_Fault_Motor_Y();
    }
    else
    {
        /* Clear the fault count for the Y-axis motor. */
        Fault_Count_Y = 0;
    }
    /* Return whether or not the fault count is greater than a specified amount. */
    return(Fault_Count_Y>Motor_Fault_Count_Max);
}

/** 
 * @brief This function checks to see if the Z-axis motor is failing. 
 * Counting and resetting the motor faults to make sure the motor is able to continue. 
 * @return Whether or not the fault count is greater than a specified amount.
 */
uint8_t DRV8830_Check_Motor_Z(void)
{
    /* Initialize the fault count for amount of motor failures. */
    static uint8_t Fault_Count_Z = 0;
    /* Read the fault register from the Z-axis motor h-bridge. */
    uint8_t Fault_Value = DRV8830_Read_Fault_Motor_Z();
    if(Fault_Value&0x01)
    {
        /* Clear the fault register for the Z-axis motor h-bridge. */
        DRV8830_Reset_Motor_Z();
        /* Increment the fault count for the Z-axis motor. */
        Fault_Count_Z++;
    }
    else if(Fault_Value)
    {
        /* Clear fault previous fault values. */
        DRV8830_Clear_Fault_Motor_Z();    
    }
    else
    {
        /* Clear the fault count for the Z-axis motor. */
        Fault_Count_Z = 0;
    }
    /* Return whether or not the fault count is greater than a specified amount. */
    return(Fault_Count_Z>Motor_Fault_Count_Max);
}

/** 
 * @brief This function checks to see if any of the 3 motors is failing.
 * @return Whether or not the fault count is greater than a specified amount for each motor.\n
 * Bit 0: Z-axis motor\n
 * Bit 1: Y-axis motor\n
 * Bit 2: X-axis motor\n
 */
uint8_t DRV8830_Check_Motors(void)
{
    /* Return the value for the fault for each of the motors. Simplifying the return value into 3 bits.*/
    return(DRV8830_Check_Motor_X()<<2|DRV8830_Check_Motor_Y()<<1|DRV8830_Check_Motor_Z()<<0);
}