/**
 * @file LSM303.c
 * @author Nicholas Sikkema
 * @version Revision: 1.5
 * @date Last Updated: 1/22/2015\n Created: 10/30/2014 4:17:54 PM
 * @brief This file is code to control the LSM303.
 */ 

#define Device_DLH 0
#define Device_DLM 1
#define Device_DLHC 2
#define Device_D 3
#define Current_Device Device_D

#include <stdint.h>
#include <math.h>
#include "../../Microcontroller/Communication/I2C_Interface.h"
#include "LSM303.h"

enum Register_Address
{
    CTRL0             = 0x1F, // D
    CTRL1             = 0x20, // D
    CTRL_REG1_A       = 0x20, // DLH, DLM, DLHC
    CTRL2             = 0x21, // D
    CTRL_REG2_A       = 0x21, // DLH, DLM, DLHC
    CTRL_REG4_A       = 0x23, // DLH, DLM, DLHC
    CTRL5             = 0x24, // D
    CTRL6             = 0x25, // D
    CTRL7             = 0x26, // D
    OUT_X_L_A         = 0x28,

    CRA_REG_M         = 0x00, // DLH, DLM, DLHC
    CRB_REG_M         = 0x01, // DLH, DLM, DLHC
    MR_REG_M          = 0x02, // DLH, DLM, DLHC

    WHO_AM_I          = 0x0F, // D

    // dummy addresses for registers in different locations on different devices;
    // the library translates these based on device type
    // value with sign flipped is used as index into translated_regs array

    OUT_X_H_M         = 1,
    OUT_X_L_M         = 2,
    OUT_Y_H_M         = 3,
    OUT_Y_L_M         = 4,
    OUT_Z_H_M         = 5,
    OUT_Z_L_M         = 6,
    // Update dummy_reg_count if registers are added here!

    // device-specific register addresses

    DLH_OUT_X_H_M     = 0x03,
    DLH_OUT_X_L_M     = 0x04,
    DLH_OUT_Y_H_M     = 0x05,
    DLH_OUT_Y_L_M     = 0x06,
    DLH_OUT_Z_H_M     = 0x07,
    DLH_OUT_Z_L_M     = 0x08,

    DLM_OUT_X_H_M     = 0x03,
    DLM_OUT_X_L_M     = 0x04,
    DLM_OUT_Z_H_M     = 0x05,
    DLM_OUT_Z_L_M     = 0x06,
    DLM_OUT_Y_H_M     = 0x07,
    DLM_OUT_Y_L_M     = 0x08,

    DLHC_OUT_X_H_M    = 0x03,
    DLHC_OUT_X_L_M    = 0x04,
    DLHC_OUT_Z_H_M    = 0x05,
    DLHC_OUT_Z_L_M    = 0x06,
    DLHC_OUT_Y_H_M    = 0x07,
    DLHC_OUT_Y_L_M    = 0x08,

    D_OUT_X_L_M       = 0x08,
    D_OUT_X_H_M       = 0x09,
    D_OUT_Y_L_M       = 0x0A,
    D_OUT_Y_H_M       = 0x0B,
    D_OUT_Z_L_M       = 0x0C,
    D_OUT_Z_H_M       = 0x0D
};

//Global Variables
static uint8_t Acc_Address = 0;
static uint8_t Mag_Address = 0;
#define Register_Count 6
static enum Register_Address Translated_Register[Register_Count+1];// index 0 not used
static float Mag_Min[3] = {-32767, -32767, -32767};
static float Mag_Max[3] = {32767, 32767, 32767};
static float Mag_From[3] = {0,0,0};
static int16_t Mag_Temp[3] = {0,0,0};
static int16_t Acc_Temp[3] = {0,0,0};
static float Forward_Heading = 0;

#define Heading_Stable_Value 2
    
#define TEST_REG_INVALID           -1
#define D_SA0_HIGH_ADDRESS         0x1D // D with SA0 high
#define D_SA0_LOW_ADDRESS          0x1E // D with SA0 low or non-D magnetometer
#define NON_D_MAG_ADDRESS          0x1E // D with SA0 low or non-D magnetometer
#define NON_D_ACC_SA0_LOW_ADDRESS  0x18 // non-D accelerometer with SA0 low
#define NON_D_ACC_SA0_HIGH_ADDRESS 0x19 // non-D accelerometer with SA0 high
#define Radians_To_Degrees_Conv    57.2957795131

#define X 0
#define Y 1
#define Z 2

#define D_WHO_ID    0x49
#define DLM_WHO_ID  0x3C

uint8_t LSM303_Test_Register(uint8_t Device_Address, uint8_t Register)
{
    uint8_t Message_Buff = I2C_Interface_Single_Read(Device_Address, Register);
    if(I2C_Interface_Not_Busy())
    {
        return Message_Buff;
    }
    else
    {
        return TEST_REG_INVALID;
    }
}

void LSM303_Initialize()
{
    // determine Device type if necessary
    #if Current_Device == Device_D
        if(LSM303_Test_Register(D_SA0_HIGH_ADDRESS, WHO_AM_I) == D_WHO_ID)
        {
            Acc_Address = D_SA0_HIGH_ADDRESS;
            Mag_Address = D_SA0_HIGH_ADDRESS;
        }
        else
        {
            Acc_Address = D_SA0_LOW_ADDRESS;
            Mag_Address = D_SA0_LOW_ADDRESS;
        }
        
        // set device addresses and translated register addresses
        Translated_Register[OUT_X_L_M] = D_OUT_X_L_M;
        Translated_Register[OUT_X_H_M] = D_OUT_X_H_M;
        Translated_Register[OUT_Y_L_M] = D_OUT_Y_L_M;
        Translated_Register[OUT_Y_H_M] = D_OUT_Y_H_M;
        Translated_Register[OUT_Z_L_M] = D_OUT_Z_L_M;
        Translated_Register[OUT_Z_H_M] = D_OUT_Z_H_M;
        
        Mag_From[0] = 1;
        Mag_From[1] = 0;
        Mag_From[2] = 0;
        
        // Accelerometer

        // 0x57 = 0b01010111
        // AFS = 0 (+/- 2 g full scale)
        I2C_Interface_Write(Acc_Address, CTRL2, 0x00);

        // 0x57 = 0b01010111
        // AODR = 0101 (50 Hz ODR);AZEN = AYEN = AXEN = 1 (all axes enabled)
        I2C_Interface_Write(Acc_Address, CTRL1, 0x57);

        // Magnetometer

        // 0x64 = 0b01100100
        // M_RES = 11 (high resolution mode);M_ODR = 001 (6.25 Hz ODR)
        I2C_Interface_Write(Mag_Address, CTRL5, 0x64);

        // 0x20 = 0b00100000
        // MFS = 01 (+/- 4 gauss full scale)
        I2C_Interface_Write(Mag_Address, CTRL6, 0x20);

        // 0x00 = 0b00000000
        // MLP = 0 (low power mode off);MD = 00 (continuous-conversion mode)
        I2C_Interface_Write(Mag_Address, CTRL7, 0x00);
    #elif Current_Device == Device_DLHC
        // set device addresses and translated register addresses
        Acc_Address = NON_D_ACC_SA0_HIGH_ADDRESS;// DLHC doesn't have SA0 but uses same acc address as DLH/DLM with SA0 high
        Mag_Address = NON_D_MAG_ADDRESS;
        Translated_Register[OUT_X_H_M] = DLHC_OUT_X_H_M;
        Translated_Register[OUT_X_L_M] = DLHC_OUT_X_L_M;
        Translated_Register[OUT_Y_H_M] = DLHC_OUT_Y_H_M;
        Translated_Register[OUT_Y_L_M] = DLHC_OUT_Y_L_M;
        Translated_Register[OUT_Z_H_M] = DLHC_OUT_Z_H_M;
        Translated_Register[OUT_Z_L_M] = DLHC_OUT_Z_L_M;
        
        Mag_From[0] = 0;
        Mag_From[1] = -1;
        Mag_From[2] = 0;
        
        // Accelerometer

        // 0x08 = 0b00001000
        // FS = 00 (+/- 2 g full scale);HR = 1 (high resolution enable)
        I2C_Interface_Write(Acc_Address, CTRL_REG4_A, 0x08);

        // 0x47 = 0b01000111
        // ODR = 0100 (50 Hz ODR);LPen = 0 (normal mode);Zen = Yen = Xen = 1 (all axes enabled)
        I2C_Interface_Write(Acc_Address, CTRL_REG1_A, 0x47);

        // Magnetometer

        // 0x0C = 0b00001100
        // DO = 011 (7.5 Hz ODR)
        I2C_Interface_Write(Mag_Address, CRA_REG_M, 0x0C);

        // 0x20 = 0b00100000
        // GN = 001 (+/- 1.3 gauss full scale)
        I2C_Interface_Write(Mag_Address, CRB_REG_M, 0x20);

        // 0x00 = 0b00000000
        // MD = 00 (continuous-conversion mode)
        I2C_Interface_Write(Mag_Address, MR_REG_M, 0x00);
    #elif Current_Device == Device_DLH
        // Device responds to address 0011000 with DLM ID;guess that it's a DLM
        if(LSM303_Test_Register(NON_D_ACC_SA0_LOW_ADDRESS, CTRL_REG1_A) != TEST_REG_INVALID)
        {
            Acc_Address = NON_D_ACC_SA0_HIGH_ADDRESS;
        }
        else
        {
            Acc_Address = NON_D_ACC_SA0_LOW_ADDRESS;
        }
        
        // set device addresses and translated register addresses
        Mag_Address = NON_D_MAG_ADDRESS;
        Translated_Register[OUT_X_H_M] = DLH_OUT_X_H_M;
        Translated_Register[OUT_X_L_M] = DLH_OUT_X_L_M;
        Translated_Register[OUT_Y_H_M] = DLH_OUT_Y_H_M;
        Translated_Register[OUT_Y_L_M] = DLH_OUT_Y_L_M;
        Translated_Register[OUT_Z_H_M] = DLH_OUT_Z_H_M;
        Translated_Register[OUT_Z_L_M] = DLH_OUT_Z_L_M;
        
        Mag_From[0] = 0;
        Mag_From[1] = -1;
        Mag_From[2] = 0;
        
        // Accelerometer

        // 0x00 = 0b00000000
        // FS = 00 (+/- 2 g full scale)
        I2C_Interface_Write(Acc_Address, CTRL_REG4_A, 0x00);

        // 0x27 = 0b00100111
        // PM = 001 (normal mode);DR = 00 (50 Hz ODR);Zen = Yen = Xen = 1 (all axes enabled)
        I2C_Interface_Write(Acc_Address, CTRL_REG1_A, 0x27);

        // Magnetometer

        // 0x0C = 0b00001100
        // DO = 011 (7.5 Hz ODR)
        I2C_Interface_Write(Mag_Address, CRA_REG_M, 0x0C);

        // 0x20 = 0b00100000
        // GN = 001 (+/- 1.3 gauss full scale)
        I2C_Interface_Write(Mag_Address, CRB_REG_M, 0x20);

        // 0x00 = 0b00000000
        // MD = 00 (continuous-conversion mode)
        I2C_Interface_Write(Mag_Address, MR_REG_M, 0x00);
    #elif Current_Device == Device_DLM
        // set device addresses and translated register addresses
        Acc_Address = NON_D_ACC_SA0_HIGH_ADDRESS;
        Mag_Address = NON_D_MAG_ADDRESS;
        Translated_Register[OUT_X_H_M] = DLM_OUT_X_H_M;
        Translated_Register[OUT_X_L_M] = DLM_OUT_X_L_M;
        Translated_Register[OUT_Y_H_M] = DLM_OUT_Y_H_M;
        Translated_Register[OUT_Y_L_M] = DLM_OUT_Y_L_M;
        Translated_Register[OUT_Z_H_M] = DLM_OUT_Z_H_M;
        Translated_Register[OUT_Z_L_M] = DLM_OUT_Z_L_M;
        
        Mag_From[0] = 0;
        Mag_From[1] = -1;
        Mag_From[2] = 0;
        
        // Accelerometer

        // 0x00 = 0b00000000
        // FS = 00 (+/- 2 g full scale)
        I2C_Interface_Write(Acc_Address, CTRL_REG4_A, 0x00);

        // 0x27 = 0b00100111
        // PM = 001 (normal mode);DR = 00 (50 Hz ODR);Zen = Yen = Xen = 1 (all axes enabled)
        I2C_Interface_Write(Acc_Address, CTRL_REG1_A, 0x27);

        // Magnetometer

        // 0x0C = 0b00001100
        // DO = 011 (7.5 Hz ODR)
        I2C_Interface_Write(Mag_Address, CRA_REG_M, 0x0C);

        // 0x20 = 0b00100000
        // GN = 001 (+/- 1.3 gauss full scale)
        I2C_Interface_Write(Mag_Address, CRB_REG_M, 0x20);

        // 0x00 = 0b00000000
        // MD = 00 (continuous-conversion mode)
        I2C_Interface_Write(Mag_Address, MR_REG_M, 0x00);
    #endif
}


void LSM303_Read_Mag(int16_t *Mag)
{
    uint8_t Readings[6] = {0,0,0,0,0,0};
    #if Current_Device == Device_D
        I2C_Interface_Read_Array(Mag_Address,Translated_Register[OUT_X_L_M]|(1<<7),Readings,6);
    #else
        I2C_Interface_Read_Array(Mag_Address,Translated_Register[OUT_X_H_M],Readings,6);
    #endif
    
    #if (Current_Device == Device_D)
        Mag[X] = (Readings[1]<<8|Readings[0]);
        Mag[Y] = (Readings[3]<<8|Readings[2]);
        Mag[Z] = (Readings[5]<<8|Readings[4]);
    #elif(Current_Device == Device_DLH)
        Mag[X] = (Readings[0]<<8|Readings[1]);
        Mag[Y] = (Readings[2]<<8|Readings[3]);
        Mag[Z] = (Readings[4]<<8|Readings[5]);
    #else
        Mag[X] = (Readings[0]<<8|Readings[1]);
        Mag[Y] = (Readings[4]<<8|Readings[5]);
        Mag[Z] = (Readings[2]<<8|Readings[3]);
    #endif
}


void LSM303_Read_Acc(int16_t *Acc)
{
    uint8_t Readings[6] = {0,0,0,0,0,0};
    I2C_Interface_Read_Array(Acc_Address,OUT_X_L_A,Readings,6);
    Acc[X] = (Readings[1]<<8|Readings[0]);
    Acc[Y] = (Readings[3]<<8|Readings[2]);
    Acc[Z] = (Readings[5]<<8|Readings[4]);
}

inline void LSM303_Array_Cross_Product(float *a, float *b, float *Output)
{
    Output[X] = (a[Y]*b[Z]) - (a[Z]*b[Y]);
    Output[Y] = (a[Z]*b[X]) - (a[X]*b[Z]);
    Output[Z] = (a[X]*b[Y]) - (a[Y]*b[X]);
}

inline float LSM303_Array_Dot_Product(float *a, float *b)
{
    return (a[X]*b[X]+a[Y]*b[Y]+a[Z]*b[Z]);
}

inline void LSM303_Array_Normalize(float *a)
{
    float Magnitude = sqrt(LSM303_Array_Dot_Product(a,a));
    a[X] /= Magnitude;
    a[Y] /= Magnitude;
    a[Z] /= Magnitude;
}

float LSM303_Heading(int16_t *Acc, int16_t *Mag)
{
    Mag[X] -= (Mag_Min[X]+Mag_Max[X])/2;
    Mag[Y] -= (Mag_Min[Y]+Mag_Max[Y])/2;
    Mag[Z] -= (Mag_Min[Z]+Mag_Max[Z])/2;
    float E[3] = {0,0,0};
    float N[3] = {0,0,0};
    float TempMag[3] = {Mag[X], Mag[Y], Mag[Z]};
    float TempAcc[3] = {Acc[X], Acc[Y], Acc[Z]};
    LSM303_Array_Cross_Product(TempMag,TempAcc,E);
    LSM303_Array_Normalize(E);
    LSM303_Array_Cross_Product(TempAcc,E,N);
    LSM303_Array_Normalize(N);
    float heading = atan2(LSM303_Array_Dot_Product(E,Mag_From),LSM303_Array_Dot_Product(N,Mag_From))*Radians_To_Degrees_Conv;
    if(heading < 0)
    {
        heading += 360;
    }
    return heading;
}

void LSM303_UpdateForwardHeading(void)
{
    LSM303_Read_Mag(Mag_Temp);
    LSM303_Read_Acc(Acc_Temp);
    Forward_Heading = LSM303_Heading(Acc_Temp, Mag_Temp);
}

float LSM303_HeadingWithOffset(float heading)
{
    float Temp_Heading = heading - Forward_Heading;
    if(Temp_Heading < 0)
    {
        Temp_Heading += 360;
    }
    return Temp_Heading;
}

uint8_t LSM303_RotationFinished(void)
{
    LSM303_Read_Mag(Mag_Temp);
    LSM303_Read_Acc(Acc_Temp);
    float Heading = LSM303_HeadingWithOffset(LSM303_Heading(Acc_Temp, Mag_Temp));
    static float Previous_Heading = 0;
    static uint8_t flag = 0;
    int Return_Value = 0;
    if((Heading<5&&Heading>-5)&&(Previous_Heading>5||Previous_Heading<-5))
    {
        if (flag == 0)
        {
            flag = 1;
        }
        else
        {
            Return_Value = 1;
        }
    }
    Previous_Heading = Heading;
    return(Return_Value);
}

uint8_t LSM303_IsStablized(void)
{
    static float Previous_Heading_Value = 1000;
    float Heading_Change = Forward_Heading-Previous_Heading_Value;
    Previous_Heading_Value = Forward_Heading;
    if(Heading_Change<Heading_Stable_Value||Heading_Change<-Heading_Stable_Value)
    {
        return(1);
    }
    else
    {
        return(0);
    }
}