/**
 * @file CLS15.c
 * @author Nicholas Sikkema
 * @version Revision: 2.0
 * @date Last Updated: 1/27/2015\n Created: 12/2/2014 4:32:25 PM
 * @brief This file is code for the photodiode readings.
 */ 

#include <stdint.h>
#include "../../Microcontroller/ADC/ADC.h"
#include "../../Microcontroller/ADC/CD4051.h"
#include "CLS15.h"

/* Static variables to hold the photodiode values. */
static uint16_t Photodiode_Value_Front = 0;
static uint16_t Photodiode_Value_Right_Front = 0;
static uint16_t Photodiode_Value_Right_Rear = 0;
static uint16_t Photodiode_Value_Left_Front = 0;
static uint16_t Photodiode_Value_Left_Rear = 0;
/* Static variables to hold the photodiode offsets. */
static uint16_t Photodiode_Value_Front_Offset = 5;
static uint16_t Photodiode_Value_Right_Front_Offset = 5;
static uint16_t Photodiode_Value_Right_Rear_Offset = 5;
static uint16_t Photodiode_Value_Left_Front_Offset = 5;
static uint16_t Photodiode_Value_Left_Rear_Offset = 5;
/* Static variables to hold the ratios for the left and right sides. */
static float left_ratio = 0;
static float right_ratio = 0;

/**
 * @brief Determines which Photodiode is reading higher and creates a ratio based on that knowledge.
 * @param Front_Value The current value for the front (left or right) photodiode.
 * @param Rear_Value The current value for the rear (left or right) photodiode.
 * @return The ratio between the front and rear photodiode values. If > 1 then it is in the front, otherwise it is in the rear.
 */
inline float CLS15_Ratio_Calculation(uint16_t Front_Value, uint16_t Rear_Value)
{
    /** Return the ratio of the front value to the rear value. */
    return((float)Front_Value/(float)Rear_Value);
}

/**
 * @brief Update the front photodiode value.
 */
void CLS15_Update_Photodiode_Value_Front(void)
{
    Photodiode_Value_Front = CD4051_Read_Single_Pin(Photodiode_front)-Photodiode_Value_Front_Offset;
}

/**
 * @brief Update the right photodiode values.
 */
void CLS15_Update_Photodiode_Value_Right(void)
{
    Photodiode_Value_Right_Front = CD4051_Read_Single_Pin(Photodiode_rightfront)-Photodiode_Value_Right_Front_Offset;
    Photodiode_Value_Right_Rear = CD4051_Read_Single_Pin(Photodiode_rightrear)-Photodiode_Value_Right_Rear_Offset;
    right_ratio = CLS15_Ratio_Calculation(Photodiode_Value_Right_Front, Photodiode_Value_Right_Rear);
}

/**
 * @brief Update the left photodiode values.
 */
void CLS15_Update_Photodiode_Value_Left(void)
{
    Photodiode_Value_Left_Front = CD4051_Read_Single_Pin(Photodiode_leftfront)-Photodiode_Value_Left_Front_Offset;
    Photodiode_Value_Left_Rear = CD4051_Read_Single_Pin(Photodiode_leftrear)-Photodiode_Value_Left_Rear_Offset;
    left_ratio = CLS15_Ratio_Calculation(Photodiode_Value_Left_Front, Photodiode_Value_Left_Rear);
}

/**
 * @brief Update all of the photodiode values.
 */
void CLS15_Update_Photodiode_Values(uint8_t Current_LED)
{
    if(Current_LED == 1)
    {
        CLS15_Update_Photodiode_Value_Right();
        CLS15_Update_Photodiode_Value_Left();
    }
    else if(Current_LED == 2)
    {
        CLS15_Update_Photodiode_Value_Front();
        CLS15_Update_Photodiode_Value_Left();
    }
    else if(Current_LED == 4)
    {
        CLS15_Update_Photodiode_Value_Front();
        CLS15_Update_Photodiode_Value_Right();
    }
    else if(Current_LED == 8)
    {
        CLS15_Update_Photodiode_Value_Front();
        CLS15_Update_Photodiode_Value_Right();
        CLS15_Update_Photodiode_Value_Left(); 
    }
    else if (Current_LED == 16)
    {
        CLS15_Update_Photodiode_Value_Front();
    }
}

/**
 * @brief Retrieve the ADC value for the front photodiode.
 * @return The ADC value for the front photodiode.
 */
uint16_t CLS15_Retrieve_Photodiode_ADC_Front(void)
{
    return(Photodiode_Value_Front);
}

/**
 * @brief Retrieve the ADC value for the right front photodiode.
 * @return The ADC value for the right front photodiode.
 */
uint16_t CLS15_Retrieve_Photodiode_ADC_Right_Front(void)
{
    return(Photodiode_Value_Right_Front);
}

/**
 * @brief Retrieve the ADC value for the right rear photodiode.
 * @return The ADC value for the right rear photodiode.
 */
uint16_t CLS15_Retrieve_Photodiode_ADC_Right_Rear(void)
{
    return(Photodiode_Value_Right_Rear);
}

/**
 * @brief Retrieve the ADC value for the left front photodiode.
 * @return The ADC value for the left front photodiode.
 */
uint16_t CLS15_Retrieve_Photodiode_ADC_Left_Front(void)
{
    return(Photodiode_Value_Left_Front);
}

/**
 * @brief Retrieve the ADC value for the left rear photodiode.
 * @return The ADC value for the left rear photodiode.
 */
uint16_t CLS15_Retrieve_Photodiode_ADC_Left_Rear(void)
{
    return(Photodiode_Value_Left_Rear);
}

/**
 * @brief Retrieve the ratio for the right photodiodes.
 * @return The ratio for the right photodiodes.
 */
float CLS15_Retrieve_Photodiode_Ratio_Right(void)
{
    return(right_ratio);
}

/**
 * @brief Retrieve the ratio for the left photodiodes.
 * @return The ratio for the left photodiodes.
 */
float CLS15_Retrieve_Photodiode_Ratio_Left(void)
{
    return(left_ratio);
}

void CLS15_Update_Photodiode_Offset_Front(void)
{
    uint16_t Temp_Value = CD4051_Read_Single_Pin(Photodiode_front);
    if(Temp_Value<Photodiode_Value_Front_Offset)
    {
        Photodiode_Value_Front_Offset = Temp_Value;
    }
}

void CLS15_Update_Photodiode_Offset_Right_Front(void)
{
    uint16_t Temp_Value = CD4051_Read_Single_Pin(Photodiode_rightfront);
    if(Temp_Value<Photodiode_Value_Front_Offset)
    {
        Photodiode_Value_Right_Front_Offset = Temp_Value;
    }
}

void CLS15_Update_Photodiode_Offset_Right_Rear(void)
{
    uint16_t Temp_Value = CD4051_Read_Single_Pin(Photodiode_rightrear);
    if(Temp_Value<Photodiode_Value_Front_Offset)
    {
        Photodiode_Value_Right_Rear_Offset = Temp_Value;
    }
}

void CLS15_Update_Photodiode_Offset_Left_Front(void)
{
    uint16_t Temp_Value = CD4051_Read_Single_Pin(Photodiode_leftfront);
    if(Temp_Value<Photodiode_Value_Front_Offset)
    {
        Photodiode_Value_Left_Front_Offset = Temp_Value;
    }
}

void CLS15_Update_Photodiode_Offset_Left_Rear(void)
{
    uint16_t Temp_Value = CD4051_Read_Single_Pin(Photodiode_leftrear);
    if(Temp_Value<Photodiode_Value_Front_Offset)
    {
        Photodiode_Value_Left_Rear_Offset = Temp_Value;
    }
}

/**
 * @brief Update the photodiode offsets.
 */
void CLS15_Update_Photodiode_Offset(void)
{
    /* Update the front photodiode offset. */
    CLS15_Update_Photodiode_Offset_Front();
    /* Update the front photodiode offset. */
    CLS15_Update_Photodiode_Offset_Left_Front();
    /* Update the front photodiode offset. */
    CLS15_Update_Photodiode_Offset_Left_Rear();
    /* Update the front photodiode offset. */
    CLS15_Update_Photodiode_Offset_Right_Front();
}