/**
 * @file XPEBBL.c
 * @author Nicholas Sikkema
 * @version Revision: 2.0
 * @date Last Updated: 1/22/2015\n Created: 12/2/2014 3:07:03 PM
 * @brief This file is code to control the 3 Watt LEDs.
 */ 

/** @brief Max number of states for the change LED function to be in. */ 
#define Cycle_State 3
/** @brief Bit position of the front LED */
#define Front_LED_Bit 5
 
#include <avr/io.h>
#include <stdint.h>
#include "XPEBBL.h"

/**
 * @brief Initialize the ports used for the LEDs.
 */ 
void XPEBBL_Initialize(void)
{
    /* Initialize the Led Ports to be an output. */
    DDRB |= (1<<DDB5)|(1<<DDB4)|(1<<DDB3);
}

/**
 * @brief Updates which led is on.
 * @param LED_Pin The pin to be set.
 * @warning This function does not check the input.
 */ 
void XPEBBL_Set_Pin(unsigned char LED_Pin)
{
    /* Clear the bits 3, 4, and 5. */
    PORTB &= 0xC7;
    /* Set the LED value in the Port. */
    PORTB |= LED_Pin;
}

/**
 * @brief This function toggles the front LED.
 */ 
uint8_t XPEBBL_Toggle_Front_LED()
{
    /* Initialize the previous value variable. */
    static uint8_t Previous_Value = 0;
    /* Initialize the current value variable. */
    uint8_t Current_Value = 0;
    /* Check if the previous value is zero. */
    if(Previous_Value == 0)
    {
        /* Enable the bit for the front LED. */
        Current_Value = 1 << Front_LED_Bit;
    }
    else
    {
        /* Clear the bit for the front LED. */
        Current_Value = 0;
    }
    /* Set the LED value. */
    XPEBBL_Set_Pin(Current_Value);
    /* Update the previous value. */
    Previous_Value = Current_Value;
    return(~(Current_Value>0));
}

/**
 * @brief Changes which LED that is currently on.
 */ 
uint8_t XPEBLL_Change_LED(void)
{
    /* Initialize the current pin variable. */
    static uint8_t Current_Pin = 0x08;
    /* Initialize the LED state variable. */
    static uint8_t LED_State = 0;
    /* Change LED State. */
    if(LED_State>Cycle_State)
    {
        /* Change the current LED that is on. */
        Current_Pin = Current_Pin << 1;
        /* Check to see if the current pin is in bounds. */
        if(Current_Pin > 0x20)
        {
            /* Reinitialize the current pin value. */
            Current_Pin = 0x08;
        }
        /* Reinitialize the state of the LED. */
        LED_State = 0;
        /* Set the LED value. */
        XPEBBL_Set_Pin(Current_Pin);
    }
    /* Turn off State. */
    else if(LED_State == Cycle_State)
    {
        /* Turn off the LEDs. */
        XPEBBL_Set_Pin(0x00);
    }
    /* Stay on State. */
    else
    {
        /* Make sure that the LED value is what is wanted. */
        XPEBBL_Set_Pin(Current_Pin);
    }
    /* Increment the state of the LEDs.*/
    LED_State++;
    /* Return the current pin. */
    return(Current_Pin>>3);
}